local E = select(2, ...):unpack()
local P, CM = E.Party, E.Comm

local pairs, ipairs, type, wipe, concat, format, gsub = pairs, ipairs, type, table.wipe, table.concat, string.format, string.gsub
local UnitIsConnected, CanInspect, CheckInteractDistance = UnitIsConnected, CanInspect, CheckInteractDistance
local GetPvpTalentInfoByID, GetTalentInfo, GetGlyphSocketInfo = GetPvpTalentInfoByID, GetTalentInfo, GetGlyphSocketInfo
local C_SpecializationInfo_GetInspectSelectedPvpTalent = C_SpecializationInfo and C_SpecializationInfo.GetInspectSelectedPvpTalent
local C_SpecializationInfo_GetPvpTalentSlotInfo = C_SpecializationInfo and C_SpecializationInfo.GetPvpTalentSlotInfo
local C_Traits_GetNodeInfo = C_Traits and C_Traits.GetNodeInfo
local C_Soulbinds_GetConduitSpellID = C_Soulbinds and C_Soulbinds.GetConduitSpellID

local InspectQueueFrame = CreateFrame("Frame")
local InspectTooltip, tooltipData
if not E.isDF then
	InspectTooltip = CreateFrame("GameTooltip", "OmniCDInspectToolTip", nil, "GameTooltipTemplate")
	InspectTooltip:SetOwner(UIParent, "ANCHOR_NONE")
end

local LibDeflate = LibStub("LibDeflate")
local INSPECT_DELAY = 2
local INSPECT_INTERVAL = 1
local INSPECT_PAUSE_TIME = 2
local INSPECT_TIMEOUT = 180
local nextInquiryTime = 0
local elapsedTime = 0
local isPaused
local queriedGUID

local queueEntries = {}
local staleEntries = {}

CM.SERIALIZATION_VERSION = E.isDF and 4 or 1

function CM:Enable()
	if self.enabled then
		return
	end

	self.AddonPrefix = E.AddOn
	C_ChatInfo.RegisterAddonMessagePrefix(self.AddonPrefix)
	self:RegisterEvent('CHAT_MSG_ADDON')
	self:RegisterEvent('PLAYER_EQUIPMENT_CHANGED')
	self:RegisterEvent('PLAYER_LEAVING_WORLD')
	if E.isWOTLKC then
		self:RegisterEvent('PLAYER_TALENT_UPDATE')
	elseif E.preCata then
		self:RegisterEvent('CHARACTER_POINTS_CHANGED')
	else
		self:RegisterUnitEvent('PLAYER_SPECIALIZATION_CHANGED', "player")

		self:RegisterEvent('COVENANT_CHOSEN')
		self:RegisterEvent('SOULBIND_ACTIVATED')
		self:RegisterEvent('SOULBIND_NODE_LEARNED')
		self:RegisterEvent('SOULBIND_NODE_UNLEARNED')
		self:RegisterEvent('SOULBIND_NODE_UPDATED')
		self:RegisterEvent('SOULBIND_CONDUIT_INSTALLED')
		self:RegisterEvent('SOULBIND_PATH_CHANGED')
		self:RegisterEvent('COVENANT_SANCTUM_RENOWN_LEVEL_CHANGED')

		self:RegisterEvent('TRAIT_CONFIG_UPDATED')
	end
	self:SetScript("OnEvent", function(self, event, ...)
		self[event](self, ...)
	end)

	self:InitInspect()
	self:InitCooldownSync()
	self.enabled = true
end

function CM:Disable()
	if not self.enabled then
		return
	end
	self:UnregisterAllEvents()

	self:DisableInspect()
	self:DesyncFromGroup()
	self.enabled = false
end

local function InspectQueueFrame_OnUpdate(_, elapsed)
	elapsedTime = elapsedTime + elapsed
	if elapsedTime > INSPECT_INTERVAL then
		CM:RequestInspect()
		elapsedTime = 0
	end
end

function CM:InitInspect()
	if self.initInspect then
		return
	end
	InspectQueueFrame:Hide()
	InspectQueueFrame:SetScript("OnUpdate", InspectQueueFrame_OnUpdate)
	self.initInspect = true
end

function CM:EnableInspect()
	if self.enabledInspect or next(queueEntries) == nil then
		return
	end
	self:RegisterEvent('INSPECT_READY')
	InspectQueueFrame:Show()
	self.enabledInspect = true
end

function CM:DisableInspect()
	if not self.enabledInspect then
		return
	end
	ClearInspectPlayer()
	self:UnregisterEvent('INSPECT_READY')
	InspectQueueFrame:Hide()

	wipe(P.pendingQueue)
	wipe(queueEntries)
	wipe(staleEntries)
	queriedGUID = nil
	isPaused = nil
	self.enabledInspect = false
end

function CM:DequeueInspect(guid, addToStale)
	if queriedGUID == guid then
		queriedGUID = nil
	end
	staleEntries[guid] = addToStale and queueEntries[guid] or nil
	queueEntries[guid] = nil
end

function CM:EnqueueInspect(force, guid)
	local addedTime = GetTime()
	if force then
		wipe(P.pendingQueue)
		wipe(queueEntries)
		wipe(staleEntries)
		for infoGUID in pairs(P.groupInfo) do
			if infoGUID == E.userGUID then
				self:InspectUser()
			else
				queueEntries[infoGUID] = addedTime
			end
		end
	elseif guid then
		if guid == E.userGUID then
			self:InspectUser()
		else
			queueEntries[guid] = addedTime
		end
	else
		local numPending = #P.pendingQueue
		if numPending == 0 then return end
		for i = numPending, 1, -1 do
			local pendingGUID = P.pendingQueue[i]
			queueEntries[pendingGUID] = addedTime
			P.pendingQueue[i] = nil
		end
	end

	if isPaused then
		nextInquiryTime = 0
		isPaused = nil
	end
	self:EnableInspect()
end

function CM:RequestInspect()
	local now = GetTime()
	if now < nextInquiryTime or UnitIsDead("player") or (InspectFrame and InspectFrame:IsShown()) then
		return
	end

	local stale = queriedGUID
	if stale then
		staleEntries[stale] = queueEntries[stale]
		queueEntries[stale] = nil
		queriedGUID = nil
	end

	if next(queueEntries) == nil then
		if next(staleEntries) then
			local copy = queueEntries
			queueEntries = staleEntries
			staleEntries = copy

			nextInquiryTime = now + INSPECT_PAUSE_TIME
			isPaused = true
		else
			self:DisableInspect()
		end
		return
	end
	isPaused = nil

	for unitGUID, addedTime in pairs(queueEntries) do
		local info = P.groupInfo[unitGUID]
		local isSyncedUnit = self.syncedGroupMembers[unitGUID]
		if info and not isSyncedUnit then
			local unit = info.unit
			local elapsed = now - addedTime
			if not UnitIsConnected(unit) or elapsed > INSPECT_TIMEOUT or info.isAdminObsForMDI then
				self:DequeueInspect(unitGUID)
			elseif E.preCata and
				not CheckInteractDistance(unit,1) or
				not CanInspect(unit) then
				staleEntries[unitGUID] = addedTime
				queueEntries[unitGUID] = nil
			else
				nextInquiryTime = now + INSPECT_DELAY
				queriedGUID = unitGUID
				NotifyInspect(unit)
				return
			end
		else
			self:DequeueInspect(unitGUID)
		end
	end
end

function CM:INSPECT_READY(guid)
	if queriedGUID == guid then
		self:InspectUnit(guid)
	end
end

local INVSLOT_INDEX = {
	INVSLOT_HEAD,
	INVSLOT_NECK,
	INVSLOT_SHOULDER,
	INVSLOT_CHEST,
	INVSLOT_WAIST,
	INVSLOT_LEGS,
	INVSLOT_FEET,
	INVSLOT_WRIST,
	INVSLOT_HAND,
	INVSLOT_FINGER1,
	INVSLOT_FINGER2,
	INVSLOT_BACK,
	INVSLOT_TRINKET1,
	INVSLOT_TRINKET2,
	INVSLOT_MAINHAND,

}
local NUM_INVSLOTS = #INVSLOT_INDEX

E.essenceData = {
	[2]  = { 293019, 298080, 298081, 298081, 294668, 298082, 298083, 298083 },
	[3]  = { 293031, 300009, 300010, 300010, 294910, 300012, 300013, 300013 },
	[4]  = { 295186, 298628, 299334, 299334, 295078, 298627, 299333, 299333 },
	[5]  = { 295258, 299336, 299338, 299338, 295246, 299335, 299337, 299337 },
	[6]  = { 295337, 299345, 299347, 299347, 295293, 299343, 299346, 299346 },
	[7]  = { 294926, 300002, 300003, 300003, 294964, 300004, 300005, 300005 },
	[12] = { 295373, 299349, 299353, 299353, 295365, 299348, 299350, 299350 },
	[13] = { 295746, 300015, 300016, 300016, 295750, 300018, 300020, 300020 },
	[14] = { 295840, 299355, 299358, 299358, 295834, 299354, 299357, 299357 },
	[15] = { 302731, 302982, 302983, 302983, 302916, 302984, 302985, 302985 },
	[16] = { 296036, 310425, 310442, 310442, 293030, 310422, 310426, 310426 },
	[17] = { 296072, 299875, 299876, 299876, 296050, 299878, 299879, 299879 },
	[18] = { 296094, 299882, 299883, 299883, 296081, 299885, 299887, 299887 },
	[19] = { 296197, 299932, 299933, 299933, 296136, 299935, 299936, 299936 },
	[20] = { 293032, 299943, 299944, 299944, 296207, 299939, 299940, 299940 },
	[21] = { 296230, 299958, 299959, 299959, 303448, 303474, 303476, 303476 },
	[22] = { 296325, 299368, 299370, 299370, 296320, 299367, 299369, 299369 },
	[23] = { 297108, 298273, 298277, 298277, 297147, 298274, 298275, 298275 },
	[24] = { 297375, 298309, 298312, 298312, 297411, 298302, 298304, 298304 },
	[25] = { 298168, 299273, 299275, 299275, 298193, 299274, 299277, 299277 },
	[27] = { 298357, 299372, 299374, 299374, 298268, 299371, 299373, 299373 },
	[28] = { 298452, 299376, 299378, 299378, 298407, 299375, 299377, 299377 },
	[32] = { 303823, 304088, 304121, 304121, 304081, 304089, 304123, 304123 },
	[33] = { 295046, 299984, 299988, 299988, 295164, 299989, 299991, 299991 },
	[34] = { 310592, 310601, 310602, 310602, 310603, 310607, 310608, 310608 },
	[35] = { 310690, 311194, 311195, 311195, 310712, 311197, 311198, 311198 },
	[36] = { 311203, 311302, 311303, 311303, 311210, 311304, 311306, 311306 },
	[37] = { 312725, 313921, 313922, 313922, 312771, 313919, 313920, 313920 },
}

CM.essencePowerIDs = {}

for essenceID, essencePowers in pairs(E.essenceData) do
	local link = E.postBFA and C_AzeriteEssence.GetEssenceHyperlink(essenceID, 1)
	if link and link ~= "" then
		link = link:match("%[(.-)%]"):gsub("%-","%%-")
		essencePowers.name = link
		essencePowers.ID = essenceID
		for i = 1, #essencePowers do
			local spellID = essencePowers[i]
			local rank1ID = essencePowers[i > 4 and 5 or 1]
			CM.essencePowerIDs[spellID] = rank1ID
		end
	end
end

function E:IsEssenceRankUpgraded(id)
	return id and id ~= CM.essencePowerIDs[id]
end

local function GetNumTooltipLines()
	if InspectTooltip then
		return InspectTooltip:NumLines()
	end
	return tooltipData and tooltipData.lines and #tooltipData.lines or 0
end

local function GetTooltipLineData(i)
	local lineData
	if tooltipData then
		lineData = tooltipData.lines[i]
		return lineData, lineData.leftText
	elseif InspectTooltip then
		lineData = _G["OmniCDInspectToolTipTextLeft" .. i]
		return lineData, lineData:GetText()
	end
end

local function GetTooltipLineTextColor(lineData)
	if not lineData then
		return 1, 1, 1
	elseif tooltipData then
		return lineData.leftColor.r, lineData.leftColor.g, lineData.leftColor.b
	elseif InspectTooltip then
		return lineData:GetTextColor()
	end
end

local ITEM_LEVEL = string.gsub(ITEM_LEVEL,"%%d","(%%d+)")

local function FindAzeriteEssencePower(info, specID, list)
	local heartOfAzerothLevel
	local majorID

	local numLines = math.min(16, GetNumTooltipLines())
	for j = 2, numLines do
		local lineData, text = GetTooltipLineData(j)
		if text and text ~= "" then
			if not heartOfAzerothLevel then
				heartOfAzerothLevel = strmatch(text, ITEM_LEVEL)
				if heartOfAzerothLevel then
					heartOfAzerothLevel = tonumber(heartOfAzerothLevel)
				end
			elseif j > 10 then
				for essenceID, essencePowers in pairs(E.essenceData) do
					if strfind(text, essencePowers.name .. "$") == 1 then
						local r, _, b = GetTooltipLineTextColor(lineData)
						local rank = 3
						if r < .01 then
							rank = 2
						elseif r > .90 then
							rank = 4
						elseif b < .01 then
							rank = 1
						end

						if not majorID and GetTooltipLineData(j - 1) == " " then
							majorID = essencePowers[rank]
							local rank1 = essencePowers[1]
							info.talentData[rank1] = "AE"
							info.talentData["essMajorRank1"] = rank1
							info.talentData["essMajorID"] = majorID
							if list then
								list[#list + 1] = majorID .. ":AE"
							end

							if E.essMajorConflict[majorID] then
								local pvpTalent = E.pvpTalentsByEssMajorConflict[specID]
								if pvpTalent then
									info.talentData[pvpTalent] = "AE"
									if list then
										list[#list + 1] = pvpTalent
									end
								end
							end
							if rank1 ~= 296325 then
								break
							end
						end

						local minorID = essencePowers[rank + 4]
						if E.essMinorStrive[minorID] then

							local mult = (90.1 - ((heartOfAzerothLevel - 117) * 0.15)) / 100
							if P.isInPvPInstance then
								mult = 0.2 + mult * 0.8
							end
							mult = math.max(0.75, math.min(0.9, mult))
							info.talentData["essStriveMult"] = mult
							if list then
								list[#list + 1] = mult .. ":ae"
							end
							return
						end
						break
					end
				end
			end
		end
	end
end

local function FindAzeritePower(info, list)
	local numLines = GetNumTooltipLines()
	for j = 10, numLines do
		local _, text = GetTooltipLineData(j)
		if text and text ~= "" and strfind(text, "^-") == 1 then
			for _, v in pairs(E.spell_cxmod_azerite) do
				if strfind(text, v.name .. "$") == 3 then
					info.talentData[v.azerite] = "A"
					if list then list[#list + 1] = v.azerite .. ":A" end
					return
				end
			end
		end
	end
end

local S_ITEM_SET_NAME = "^" .. ITEM_SET_NAME:gsub("([%(%)])", "%%%1"):gsub("%%%d?$?d", "(%%d+)"):gsub("%%%d?$?s", "(.+)") .. "$"

local function FindSetBonus(info, specBonus, list)
	local bonusID, numRequired = specBonus[1], specBonus[2]
	local numLines = GetNumTooltipLines()
	for j = 10, numLines do
		local _, text = GetTooltipLineData(j)
		if text and text ~= "" then
			local name, numEquipped, numFullSet = strmatch(text, S_ITEM_SET_NAME)
			if name and numEquipped and numFullSet then
				numEquipped = tonumber(numEquipped)
				if numEquipped and numEquipped >= numRequired then
					info.talentData[bonusID] = "S"
					if list then list[#list + 1] = bonusID .. ":S" end
				end
				return bonusID
			end
		end
	end
end

local function FindCraftedRuneforgeLegendary(info, itemLink, list)
	local _,_,_,_,_,_,_,_,_,_,_,_,_, numBonusIDs, bonusIDs = strsplit(":", itemLink, 15)
	numBonusIDs = tonumber(numBonusIDs)
	if numBonusIDs and bonusIDs then
		local t = { strsplit(":", bonusIDs, numBonusIDs + 1) }
		for j = 1, numBonusIDs do
			local bonusID = t[j]
			bonusID = tonumber(bonusID)
			local runeforgeDescID = E.runeforge_bonus_to_descid[bonusID]
			if runeforgeDescID then
				if type(runeforgeDescID) == "table" then
					for _, descID in pairs(runeforgeDescID) do
						info.talentData[descID] = "R"
						if list then list[#list + 1] = descID .. ":R" end
					end
				else
					info.talentData[runeforgeDescID] = "R"
					if list then list[#list + 1] = runeforgeDescID .. ":R" end
				end
				return
			end
		end
	end
end

local runeforgeBaseItems = {
	[1]  = { 173245, 172317, 172325, 171415 },
	[2]  = { 178927, 178927, 178927, 178927 },
	[3]  = { 173247, 172319, 172327, 171417 },
	[5]  = { 173241, 172314, 172322, 171412 },
	[6]  = { 173248, 172320, 172328, 171418 },
	[7]  = { 173246, 172318, 172326, 171416 },
	[8]  = { 173243, 172315, 172323, 171413 },
	[9]  = { 173249, 172321, 172329, 171419 },
	[10] = { 173244, 172316, 172324, 171414 },
	[11] = { 178926, 178926, 178926, 178926 },
	[12] = { 178926, 178926, 178926, 178926 },
	[15] = { 173242, 173242, 173242, 173242 },
}

--[[
if we're separating player insepct:
	local itemID = GetInventoryItemID(unit, slotID)
	local itemLink = GetInventoryItemLink(unit, slotID)
	local itemLocation = ItemLocation:CreateFromEquipmentSlot(slotID)
	local isRuenforgeBaseItem = C_LegendaryCrafting.IsValidRuneforgeBaseItem(itemLocation)
	local isRuneforgeLegendary = C_LegendaryCrafting.IsRuneforgeLegendary(itemLocation)
]]
local function GetEquippedItemData(info, unit, specID, list)
	local moveToStale
	local numRuneforge = 0
	local numTierSetBonus = 0
	local foundTierSpecBonus
	if list then list[#list + 1] = "^M" end
	local cloak --[[ TODO: temp fix: Undulating Sporecloak ]]

	for i = 1, NUM_INVSLOTS do
		local slotID = INVSLOT_INDEX[i]
		local itemLink = GetInventoryItemLink(unit, slotID)
		if itemLink then
			local itemID, _,_,_,_,_, subclassID = GetItemInfoInstant(itemLink)
			if itemID then
				if i <= 12 then
					local tierSetBonus = E.item_set_bonus[itemID]
					local equipBonusID = E.item_equip_bonus[itemID]
					subclassID = subclassID == 0 and 1 or subclassID
					local unityRuneforgeLegendary = E.item_unity[itemID]
					local isCraftedRuneforgeLegendary = numRuneforge <= 2
						and runeforgeBaseItems[slotID]
						and itemID == runeforgeBaseItems[slotID][subclassID]
					if InspectTooltip then
						InspectTooltip:SetInventoryItem(unit, slotID)
					else --[[https://wowpedia.fandom.com/wiki/Patch_10.0.2/API_changes#Tooltip_Changes]]
						tooltipData = C_TooltipInfo.GetInventoryItem(unit, slotID)
						if tooltipData then
							TooltipUtil.SurfaceArgs(tooltipData)
							for _, line in ipairs(tooltipData.lines) do
							    TooltipUtil.SurfaceArgs(line)
							end
						end
					end
					if equipBonusID then
						info.talentData[equipBonusID] = true
						if list then list[#list + 1] = equipBonusID .. ":S" end
					end
					if tierSetBonus then
						local specBonus = E.preCata and tierSetBonus or tierSetBonus[specID]
						if specBonus and numTierSetBonus < 2 and specBonus[1] ~= foundTierSpecBonus then
							foundTierSpecBonus = FindSetBonus(info, specBonus, list)
							if foundTierSpecBonus then
								numTierSetBonus = numTierSetBonus + 1
							end
						end

					elseif isCraftedRuneforgeLegendary then
						FindCraftedRuneforgeLegendary(info, itemLink, list)
						numRuneforge = numRuneforge + 1
					elseif unityRuneforgeLegendary then
						if type(unityRuneforgeLegendary) == "table" then
							for _, runeforgeDescID in pairs(unityRuneforgeLegendary) do
								info.talentData[runeforgeDescID] = "R"
								if list then list[#list + 1] = runeforgeDescID .. ":R" end
							end
						else
							info.talentData[unityRuneforgeLegendary] = "R"
							if list then list[#list + 1] = unityRuneforgeLegendary .. ":R" end
						end
						numRuneforge = numRuneforge + 1
					elseif itemID == 158075 then
						FindAzeriteEssencePower(info, specID, list)
					elseif C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItemByID(itemLink) then
						FindAzeritePower(info, list)
					end
					if i == 12 then
						cloak = E.item_merged[itemID] or itemID
						info.itemData[cloak] = true
					end
					if InspectTooltip then
						InspectTooltip:ClearLines()
					end
				else
					itemID = E.item_merged[itemID] or itemID
					info.itemData[itemID] = true
					if list then list[#list + 1] = itemID end
				end
			elseif not moveToStale then
				moveToStale = true
			end
		end
		if list and i == 12 then
			list[#list + 1] = cloak and ("^E," .. cloak) or "^E"
		end
	end

	return moveToStale
end


local treeNodeSpellIDs = {
	[850] = {[386628]=179,[871]=3,[316440]=76,[382549]=85,[383442]=119,[386630]=180,[12294]=13,[383762]=125,[394855]=222,[248621]=59,[383317]=115,[390713]=200,[46968]=23,[390140]=193,[386634]=181,[215568]=53,[281001]=72,[384404]=151,[203201]=50,[384277]=145,[383959]=133,[382940]=94,[184364]=37,[388933]=186,[388807]=184,[384090]=140,[845]=2,[215571]=54,[275334]=64,[260643]=60,[386196]=170,[382946]=95,[184367]=38,[386071]=167,[390725]=201,[190456]=41,[383459]=120,[386328]=175,[184783]=39,[397103]=226,[386011]=163,[12950]=15,[386394]=177,[384036]=134,[384100]=141,[202316]=45,[202603]=47,[203177]=49,[64382]=24,[385059]=154,[383082]=101,[400803]=229,[97462]=27,[29725]=21,[383338]=116,[385952]=162,[382956]=100,[262150]=61,[400314]=228,[386208]=173,[316402]=74,[202095]=42,[400205]=227,[118038]=32,[383468]=121,[382767]=88,[383341]=117,[382895]=89,[23920]=19,[382258]=82,[383852]=127,[383916]=131,[316405]=75,[390674]=198,[386357]=176,[396719]=224,[384041]=135,[382260]=83,[383854]=128,[385703]=157,[385512]=155,[394312]=220,[384110]=142,[208154]=52,[394062]=217,[394307]=218,[394311]=219,[383155]=108,[383219]=109,[393967]=216,[152278]=35,[393965]=215,[382939]=92,[392966]=213,[382948]=96,[280392]=69,[315720]=73,[3411]=7,[390354]=194,[392790]=209,[386027]=164,[262161]=62,[383605]=123,[388004]=182,[385008]=153,[383287]=110,[12323]=14,[392777]=208,[392536]=207,[316733]=77,[107570]=29,[391997]=206,[386030]=165,[391683]=205,[391572]=204,[386477]=178,[382461]=84,[385840]=159,[6572]=11,[376079]=81,[280001]=68,[390675]=199,[23881]=18,[382954]=99,[389603]=189,[383292]=111,[262231]=63,[275336]=65,[334779]=78,[383293]=112,[392792]=210,[167105]=36,[390138]=192,[390135]=191,[383103]=103,[202168]=44,[280721]=70,[383295]=113,[384124]=144,[390563]=196,[346002]=80,[389308]=188,[389306]=187,[388903]=185,[6343]=9,[202743]=48,[388049]=183,[103827]=28,[396749]=225,[384063]=137,[386285]=174,[386164]=169,[386034]=166,[385888]=161,[385843]=160,[385704]=158,[385571]=156,[401150]=230,[394329]=221,[384361]=148,[384318]=146,[206315]=51,[391270]=202,[85288]=26,[390123]=190,[384067]=138,[391271]=203,[383430]=118,[382896]=90,[6544]=10,[383877]=129,[227847]=55,[393950]=214,[236279]=58,[107574]=31,[384042]=136,[392931]=211,[12975]=16,[275339]=67,[18499]=17,[383154]=107,[383703]=124,[383115]=106,[383922]=132,[772]=1,[384072]=139,[202163]=43,[202560]=46,[81099]=25,[5246]=8,[1719]=6,[7384]=12,[383486]=122,[383848]=126,[1160]=4,[383297]=114,[392936]=212,[29838]=22,[1161]=5,[382900]=91,[390642]=197,[382953]=98,[383885]=130,[384969]=152,[382764]=87,[335077]=79,[228920]=57,[275338]=66,},
	[852] = {[193539]=35,[394023]=158,[245687]=50,[344363]=70,[400783]=165,[382742]=142,[319949]=66,[386823]=154,[121411]=27,[319951]=67,[382746]=143,[31230]=13,[381982]=116,[385616]=151,[196922]=38,[381664]=98,[382238]=123,[108216]=26,[343160]=69,[5938]=6,[381985]=117,[196924]=39,[14161]=9,[319066]=65,[57934]=18,[255989]=52,[193546]=36,[381797]=101,[36554]=15,[2094]=2,[381798]=102,[61329]=20,[255544]=51,[378803]=75,[354897]=71,[381800]=104,[381673]=100,[378996]=78,[392384]=155,[277953]=60,[381802]=106,[426555]=176,[382504]=126,[378807]=76,[272026]=58,[382505]=127,[382506]=128,[360194]=72,[378427]=73,[35551]=14,[91023]=23,[382508]=130,[423054]=168,[79134]=22,[378813]=77,[382511]=131,[379005]=79,[381619]=82,[426563]=177,[382512]=132,[381620]=83,[382513]=133,[256188]=55,[381621]=84,[5761]=5,[382514]=134,[200758]=45,[382515]=135,[79008]=21,[381623]=86,[378436]=74,[14062]=8,[382517]=136,[382518]=137,[381626]=88,[381627]=89,[5277]=4,[423703]=174,[58423]=19,[152152]=30,[381629]=90,[428486]=182,[381630]=91,[1776]=1,[31224]=12,[381822]=107,[381631]=92,[382524]=139,[196938]=42,[280716]=62,[382015]=121,[108208]=24,[51690]=17,[394320]=160,[381634]=94,[428387]=181,[319032]=64,[384631]=145,[328085]=68,[280719]=63,[382528]=141,[200733]=44,[382017]=122,[426594]=179,[381622]=85,[108209]=25,[381637]=95,[424044]=175,[423701]=173,[423683]=172,[423662]=171,[423647]=170,[256165]=53,[423136]=169,[381894]=115,[193531]=33,[421975]=167,[400804]=166,[381640]=96,[51667]=16,[395422]=164,[394983]=163,[257505]=56,[382507]=129,[394321]=161,[394309]=159,[385722]=153,[393972]=157,[393970]=156,[231719]=48,[385627]=152,[381828]=108,[14983]=11,[385424]=149,[381669]=99,[196976]=43,[383281]=144,[121471]=28,[382525]=140,[382523]=138,[381632]=93,[394332]=162,[196937]=41,[277925]=59,[279876]=61,[385408]=148,[382245]=124,[426591]=178,[381542]=80,[381839]=109,[381543]=81,[428377]=180,[256170]=54,[13750]=7,[14190]=10,[381989]=119,[381988]=118,[381990]=120,[196861]=37,[235484]=49,[231691]=47,[185313]=31,[381801]=105,[193537]=34,[381885]=114,[381878]=113,[381652]=97,[2823]=3,[381877]=112,[185314]=32,[385478]=150,[382503]=125,[381799]=103,[381845]=110,[381624]=87,[200806]=46,[271877]=57,[381846]=111,[131511]=29,},
	[854] = {[391409]=121,[389688]=81,[320331]=43,[203550]=9,[207407]=18,[389693]=82,[389694]=83,[389695]=84,[320654]=56,[390142]=109,[389696]=85,[258887]=36,[389824]=100,[209258]=23,[278326]=41,[207697]=21,[389763]=96,[388106]=70,[207347]=16,[388107]=71,[320341]=44,[227174]=30,[388108]=72,[203555]=10,[390213]=114,[388111]=73,[389705]=86,[388112]=74,[258860]=33,[388113]=75,[370965]=69,[390154]=111,[389708]=87,[320412]=49,[258925]=37,[427640]=126,[320413]=50,[388116]=77,[326853]=59,[198013]=5,[389711]=88,[320415]=51,[390158]=112,[320416]=52,[213410]=27,[393029]=122,[320418]=53,[389715]=89,[428603]=134,[207387]=17,[247454]=32,[205411]=13,[389781]=97,[389718]=90,[389846]=101,[207739]=22,[389783]=98,[207548]=19,[389720]=91,[389976]=104,[389849]=102,[389977]=105,[391189]=118,[389978]=106,[389724]=93,[390808]=115,[218612]=29,[428232]=130,[347461]=68,[232893]=31,[389729]=94,[179057]=1,[389732]=95,[204909]=12,[428557]=133,[328725]=60,[320374]=46,[196718]=4,[202137]=7,[389799]=99,[428241]=131,[207684]=20,[320313]=42,[202138]=8,[258876]=34,[389985]=107,[212084]=25,[428607]=135,[263642]=38,[428492]=132,[320635]=55,[427794]=129,[196555]=3,[198793]=6,[320387]=48,[343311]=67,[391165]=116,[204021]=11,[389997]=108,[213010]=26,[390163]=113,[336639]=61,[209281]=24,[391397]=120,[343206]=65,[320421]=54,[395446]=125,[389220]=79,[343207]=66,[393822]=123,[391275]=119,[388114]=76,[263648]=39,[343017]=64,[320386]=47,[268175]=40,[206416]=14,[389721]=92,[393827]=124,[320361]=45,[320770]=57,[389958]=103,[258881]=35,[388118]=78,[217832]=28,[391178]=117,[427775]=127,[390152]=110,[427776]=128,[429211]=136,[183782]=2,[206476]=15,[321028]=58,[389687]=80,[343014]=63,[342817]=62,},
	[781] = {[383695]=93,[388604]=138,[322113]=73,[388477]=128,[388031]=121,[124502]=43,[383697]=94,[386949]=106,[383698]=95,[391412]=171,[393516]=189,[325177]=84,[383700]=96,[322118]=74,[388674]=143,[394093]=190,[196730]=54,[322120]=75,[388548]=134,[198898]=61,[231602]=66,[122783]=39,[388551]=135,[388615]=139,[322507]=76,[388681]=144,[388809]=150,[116680]=23,[388491]=129,[322510]=77,[388874]=161,[152173]=47,[195300]=51,[196607]=52,[115310]=14,[137639]=46,[388813]=153,[197915]=60,[386965]=107,[383714]=98,[343743]=91,[152175]=48,[124682]=44,[115008]=5,[116841]=25,[196737]=56,[115869]=19,[422031]=207,[220357]=65,[394110]=191,[388564]=136,[325197]=85,[387991]=118,[387035]=108,[274963]=70,[383785]=99,[196740]=57,[414132]=205,[325201]=86,[387230]=112,[115313]=15,[432181]=209,[418359]=206,[414131]=204,[122278]=35,[388505]=130,[386276]=102,[113656]=4,[280515]=72,[400629]=201,[123904]=40,[132578]=45,[400089]=200,[400053]=199,[388509]=131,[116844]=26,[122470]=38,[388701]=147,[388811]=151,[399226]=197,[397768]=196,[397251]=195,[388511]=132,[121817]=32,[115203]=13,[388193]=125,[393357]=186,[387046]=109,[392993]=183,[394923]=193,[157411]=49,[101545]=1,[394123]=192,[393460]=188,[323999]=80,[391383]=170,[122280]=36,[124081]=42,[191837]=50,[393400]=187,[393098]=185,[322960]=79,[396166]=194,[392991]=182,[392979]=176,[388740]=148,[388517]=133,[123986]=41,[387625]=115,[388856]=160,[389577]=164,[392983]=178,[197895]=58,[392958]=174,[115173]=9,[392970]=175,[392982]=177,[392910]=173,[392900]=172,[383994]=100,[325153]=83,[391370]=169,[388593]=137,[116847]=28,[107428]=3,[324312]=81,[389579]=166,[389578]=165,[392985]=179,[387184]=110,[389575]=163,[389574]=162,[392986]=180,[115151]=8,[202424]=62,[325093]=82,[388849]=158,[388848]=157,[391330]=168,[388682]=145,[388814]=154,[388812]=152,[399491]=198,[388779]=149,[392989]=181,[388686]=146,[115175]=10,[388846]=155,[115176]=11,[337209]=89,[280197]=71,[388847]=156,[197900]=59,[115315]=16,[264348]=67,[116849]=29,[344359]=92,[121253]=31,[387765]=117,[328669]=87,[115399]=18,[388212]=126,[387256]=113,[328670]=88,[392994]=184,[116705]=24,[386937]=103,[115078]=6,[322740]=78,[405044]=203,[388023]=120,[115396]=17,[388661]=141,[218164]=64,[116095]=20,[404408]=202,[388047]=123,[386941]=105,[343655]=90,[388854]=159,[388038]=122,[383707]=97,[196736]=55,[388020]=119,[388664]=142,[388218]=127,[387638]=116,[196725]=53,[387276]=114,[115098]=7,[101643]=2,[116645]=22,[387219]=111,[119582]=30,[122281]=37,[274909]=69,[389942]=167,[274586]=68,[115181]=12,},
	[658] = {[190447]=38,[382293]=104,[416719]=166,[382103]=97,[235870]=60,[383634]=123,[235711]=59,[389627]=154,[382424]=106,[382297]=105,[378919]=90,[45438]=17,[382490]=109,[417489]=168,[30449]=11,[384276]=139,[108853]=22,[386763]=150,[112965]=24,[417491]=169,[382110]=98,[382493]=110,[417493]=170,[391102]=157,[205020]=41,[270233]=67,[416094]=162,[205021]=42,[11366]=6,[383391]=118,[365350]=83,[108839]=21,[343230]=82,[110959]=23,[383967]=133,[382820]=113,[205023]=44,[384861]=147,[31661]=14,[384033]=135,[382440]=107,[205024]=45,[390218]=156,[382569]=111,[11426]=7,[205025]=46,[342344]=79,[383782]=129,[382826]=114,[205026]=47,[384612]=143,[12982]=10,[389713]=155,[205027]=48,[378749]=88,[157980]=34,[475]=1,[383659]=124,[205028]=49,[203275]=40,[236457]=61,[380154]=94,[157981]=35,[383661]=125,[383980]=134,[205029]=50,[384683]=145,[321507]=72,[205030]=51,[384174]=137,[84714]=20,[378947]=91,[378756]=89,[383665]=126,[387044]=152,[383092]=115,[381244]=95,[235313]=57,[383476]=119,[383604]=122,[155148]=31,[342231]=76,[383860]=131,[383669]=127,[155149]=32,[382268]=100,[321387]=70,[382270]=101,[384374]=140,[281482]=69,[382144]=99,[416506]=163,[44614]=16,[378448]=87,[1463]=3,[418733]=171,[414659]=159,[386539]=149,[384187]=138,[416715]=165,[205037]=53,[416714]=164,[384060]=136,[414660]=160,[414379]=158,[387807]=153,[386828]=151,[113724]=25,[417467]=167,[385167]=148,[384858]=146,[235224]=56,[378198]=85,[384651]=144,[153595]=29,[264354]=65,[414664]=161,[153561]=28,[383886]=132,[321526]=73,[342245]=77,[383810]=130,[383676]=128,[236628]=62,[343183]=80,[157642]=33,[56377]=19,[153626]=30,[321752]=75,[55342]=18,[212653]=54,[5143]=5,[157997]=36,[205022]=43,[235450]=58,[759]=2,[342249]=78,[190319]=37,[384452]=141,[383489]=120,[236662]=63,[278309]=68,[205036]=52,[383243]=117,[384581]=142,[231564]=55,[321745]=74,[376103]=84,[381706]=96,[44457]=15,[321420]=71,[383499]=121,[379993]=93,[378406]=86,[2948]=4,[117216]=27,[382289]=102,[12051]=8,[30455]=12,[382481]=108,[382800]=112,[269644]=66,[12472]=9,[379049]=92,[257541]=64,[343222]=81,[114923]=26,[199786]=39,[382292]=103,[383121]=116,[31589]=13,},
	[774] = {[199532]=41,[259489]=57,[385737]=144,[268501]=75,[187707]=30,[131894]=20,[56315]=14,[321287]=87,[385739]=145,[187708]=31,[155228]=23,[53351]=12,[321290]=88,[392053]=162,[120360]=18,[194595]=37,[388039]=149,[120679]=19,[265895]=72,[400472]=168,[320976]=84,[384790]=138,[388042]=150,[260228]=59,[259495]=58,[392060]=164,[213691]=48,[388045]=151,[360952]=99,[266921]=73,[389449]=155,[378740]=113,[53480]=13,[217200]=49,[147362]=22,[257044]=52,[378743]=114,[378745]=115,[391559]=161,[378937]=129,[193455]=34,[385695]=141,[257620]=53,[191384]=33,[388056]=152,[378940]=130,[257621]=54,[269737]=76,[378750]=116,[260331]=66,[343242]=94,[19577]=6,[271014]=79,[236776]=51,[360966]=100,[343244]=95,[199483]=38,[1513]=1,[378244]=111,[389019]=154,[288613]=82,[273887]=81,[390231]=160,[343248]=97,[2643]=2,[406425]=170,[186289]=27,[19434]=4,[201430]=43,[260367]=67,[260240]=60,[34477]=10,[271788]=80,[378442]=112,[260241]=61,[186387]=28,[424558]=174,[264332]=70,[269751]=77,[378955]=135,[359844]=98,[407412]=172,[407404]=171,[270581]=78,[19801]=7,[378910]=125,[264735]=71,[260243]=62,[378766]=118,[393933]=166,[393344]=165,[19574]=5,[378002]=102,[389882]=159,[389880]=158,[378950]=132,[389865]=156,[388057]=153,[378769]=120,[378948]=131,[260404]=68,[115939]=17,[385810]=146,[378961]=136,[385718]=143,[260309]=65,[385709]=142,[187698]=29,[378771]=121,[321460]=90,[321014]=85,[378888]=122,[378007]=104,[384799]=139,[193532]=35,[378962]=137,[378953]=134,[378951]=133,[389866]=157,[386870]=147,[405804]=169,[321530]=92,[193533]=36,[257944]=55,[259387]=56,[199921]=42,[321018]=86,[109215]=15,[203415]=44,[343247]=96,[34026]=9,[212436]=47,[162488]=24,[212431]=46,[378905]=123,[185789]=25,[378934]=128,[267116]=74,[378916]=127,[378205]=107,[378014]=106,[321468]=91,[199528]=39,[260248]=63,[378907]=124,[109248]=16,[375891]=101,[378207]=108,[400456]=167,[199530]=40,[385539]=140,[386878]=148,[231548]=50,[378767]=119,[5116]=3,[378209]=109,[378765]=117,[190925]=32,[186270]=26,[378210]=110,[263186]=69,[294029]=83,[424557]=173,[321293]=89,[378004]=103,[204089]=45,[378913]=126,[342049]=93,[260285]=64,[378010]=105,},
	[795] = {[377065]=132,[373049]=108,[139]=1,[62618]=21,[109186]=34,[193157]=46,[341240]=90,[391284]=179,[414553]=194,[373180]=111,[193063]=43,[391095]=167,[373054]=109,[198068]=55,[372991]=104,[373310]=119,[372354]=97,[321291]=85,[368275]=94,[238136]=75,[390972]=160,[199855]=57,[235587]=71,[200174]=59,[391228]=177,[375994]=131,[372616]=99,[424509]=200,[64843]=24,[373446]=122,[391233]=178,[373065]=110,[200209]=62,[390980]=162,[373003]=105,[265202]=80,[228260]=69,[132157]=41,[193134]=44,[407430]=190,[390919]=156,[108920]=30,[263165]=78,[108968]=33,[314867]=84,[372370]=98,[391368]=181,[238558]=76,[47536]=17,[204197]=63,[390668]=141,[215768]=68,[596]=3,[196707]=51,[372309]=96,[288733]=83,[373202]=112,[390670]=142,[213634]=67,[155271]=42,[200183]=60,[377349]=133,[321377]=86,[372760]=100,[34861]=15,[81749]=28,[335467]=88,[421557]=198,[421453]=197,[32375]=10,[32379]=11,[406788]=189,[64901]=25,[392988]=186,[199484]=56,[373273]=117,[73510]=27,[390996]=165,[409044]=193,[391124]=170,[9484]=6,[123040]=40,[271466]=81,[390615]=136,[196489]=49,[341273]=91,[373212]=113,[391381]=182,[33206]=13,[415479]=196,[406777]=188,[391209]=176,[415416]=195,[392507]=185,[373466]=126,[391387]=183,[377422]=134,[88625]=29,[393870]=187,[391208]=175,[391186]=174,[391161]=173,[122121]=39,[373280]=118,[390684]=144,[391154]=172,[375767]=128,[205385]=66,[372835]=101,[391151]=171,[373218]=114,[390622]=138,[390686]=146,[10060]=7,[605]=4,[391112]=169,[391109]=168,[391090]=166,[390994]=164,[390992]=163,[390977]=161,[390967]=159,[373221]=115,[108942]=31,[390689]=147,[2050]=5,[200128]=58,[390881]=155,[390767]=151,[197419]=54,[47515]=16,[204883]=64,[375901]=130,[390786]=153,[372969]=102,[390770]=152,[390947]=157,[390705]=150,[390693]=149,[390691]=148,[110744]=35,[390685]=145,[33076]=12,[373035]=106,[390676]=143,[390667]=140,[372972]=103,[375888]=129,[390620]=137,[15487]=9,[263716]=79,[390632]=139,[238100]=73,[373481]=127,[64129]=23,[528]=2,[238063]=72,[196985]=52,[246287]=77,[391399]=184,[377438]=135,[373457]=125,[55676]=20,[194509]=47,[390954]=158,[373456]=124,[373450]=123,[238135]=74,[373042]=107,[73325]=26,[372307]=95,[108945]=32,[341167]=89,[373223]=116,[391339]=180,[196704]=50,[15286]=8,[121536]=38,[196437]=48,[407469]=191,[421558]=199,[373427]=121,[47585]=18,[407470]=192,[200199]=61,[120644]=37,[390832]=154,[193155]=45,[64044]=22,[34433]=14,[341997]=93,[322115]=87,[341491]=92,[120517]=36,[205364]=65,[231682]=70,[47788]=19,[197045]=53,[280749]=82,},
	[793] = {[203962]=102,[390772]=167,[158477]=69,[113043]=55,[391347]=173,[382550]=153,[158478]=70,[33891]=25,[371905]=138,[203964]=103,[155578]=64,[370695]=137,[132469]=60,[131768]=58,[203965]=104,[29166]=21,[202021]=90,[383192]=157,[155675]=66,[394094]=228,[400278]=238,[155835]=67,[102558]=45,[16974]=16,[102351]=41,[383195]=158,[378988]=152,[202342]=93,[382559]=155,[391548]=178,[377842]=149,[197625]=81,[79577]=36,[102543]=44,[197721]=86,[384665]=161,[392124]=195,[197626]=82,[231063]=115,[426702]=250,[194153]=74,[417712]=248,[203651]=100,[202918]=99,[417710]=247,[231032]=112,[231064]=116,[192081]=73,[102560]=46,[202345]=94,[377210]=145,[415169]=246,[345208]=135,[197628]=85,[102401]=43,[391045]=170,[319454]=129,[372119]=140,[289237]=126,[405300]=244,[106832]=49,[391174]=172,[382552]=154,[202028]=91,[403521]=242,[202347]=95,[5217]=12,[372943]=143,[400636]=241,[99]=1,[16931]=15,[400533]=240,[108299]=54,[400320]=239,[370586]=136,[372945]=144,[203974]=105,[391700]=179,[328767]=133,[207385]=110,[400222]=236,[238049]=118,[200851]=88,[400158]=235,[391881]=184,[400142]=234,[191034]=72,[202031]=92,[274837]=121,[16864]=14,[400129]=232,[392116]=194,[394121]=230,[274902]=122,[394115]=229,[279620]=123,[106898]=51,[2782]=9,[194223]=75,[377796]=146,[5211]=11,[393611]=213,[394048]=224,[394046]=223,[393991]=222,[50334]=30,[200854]=89,[197061]=76,[393960]=221,[231040]=113,[393958]=220,[159286]=71,[205636]=108,[135288]=61,[327541]=130,[393940]=217,[108238]=53,[78674]=34,[393771]=216,[377801]=147,[48484]=27,[393618]=214,[394058]=225,[48438]=26,[384668]=163,[88747]=38,[6807]=13,[210706]=111,[393414]=211,[393371]=210,[102342]=40,[372618]=142,[392378]=208,[78675]=35,[202770]=98,[392356]=207,[102693]=47,[392325]=206,[383352]=160,[2908]=10,[392315]=205,[22570]=18,[124974]=57,[392302]=204,[145205]=63,[93402]=39,[1822]=5,[102359]=42,[393427]=212,[392256]=201,[394065]=226,[391947]=187,[202430]=97,[1079]=4,[404521]=243,[392160]=196,[400059]=231,[377847]=150,[319439]=128,[274281]=120,[392410]=209,[391709]=180,[384721]=164,[203953]=101,[391951]=189,[392220]=199,[391888]=186,[391875]=183,[391872]=182,[392221]=200,[328025]=132,[400254]=237,[106839]=50,[391528]=177,[383191]=156,[390902]=169,[390864]=168,[377811]=148,[384667]=162,[106951]=52,[114107]=56,[391969]=190,[392288]=202,[33873]=24,[385786]=165,[50464]=31,[383197]=159,[391078]=171,[392162]=197,[231050]=114,[378986]=151,[392099]=193,[390378]=166,[391972]=191,[372567]=141,[406890]=245,[371999]=139,[252216]=119,[204053]=106,[288826]=125,[343647]=134,[236068]=117,[301768]=127,[24858]=20,[33786]=23,[393760]=215,[102793]=48,[2637]=6,[392167]=198,[22842]=19,[391785]=181,[740]=2,[204066]=107,[18562]=17,[394081]=227,[327993]=131,[393954]=218,[391978]=192,[145108]=62,[200390]=87,[80313]=37,[285381]=124,[61336]=33,[421432]=249,[393956]=219,[400140]=233,[207383]=109,[132158]=59,[774]=3,[197073]=77,[202425]=96,[197524]=78,[158476]=68,[33763]=22,[392301]=203,[155580]=65,},
	[750] = {[195292]=46,[376938]=121,[194878]=41,[81136]=29,[377640]=143,[277234]=85,[276023]=82,[49143]=14,[377514]=134,[253593]=76,[111673]=32,[207311]=68,[391477]=165,[51271]=22,[194912]=42,[108199]=31,[207057]=56,[390268]=156,[194913]=43,[206930]=50,[377073]=124,[51128]=21,[374715]=115,[195679]=47,[206931]=51,[391546]=167,[221562]=74,[219809]=72,[374717]=116,[377076]=125,[49184]=15,[377587]=138,[194916]=44,[390275]=158,[207061]=57,[376251]=119,[194662]=38,[50842]=19,[205723]=48,[207317]=69,[343294]=100,[207126]=59,[55233]=24,[206967]=53,[390279]=159,[377083]=126,[374598]=114,[48707]=9,[55090]=23,[49998]=18,[221536]=73,[281208]=87,[47528]=5,[206970]=54,[205727]=49,[207289]=67,[47568]=7,[374030]=105,[377537]=135,[57330]=25,[377092]=127,[206940]=52,[219786]=71,[273946]=77,[221699]=75,[377668]=145,[279302]=86,[391566]=168,[374737]=117,[316916]=93,[212552]=70,[206974]=55,[390166]=151,[377098]=128,[207230]=63,[273952]=78,[391571]=169,[195182]=45,[273953]=79,[152279]=35,[77575]=28,[207200]=62,[207264]=64,[49530]=17,[48263]=8,[152280]=37,[374747]=118,[391386]=161,[390175]=152,[374111]=107,[327574]=99,[374049]=106,[391517]=166,[377655]=144,[392950]=171,[392566]=170,[316803]=90,[305392]=89,[390161]=150,[391398]=163,[391395]=162,[390283]=160,[390270]=157,[274156]=80,[51052]=20,[207269]=65,[390236]=154,[207167]=61,[373926]=103,[207142]=60,[194679]=39,[391458]=164,[377047]=122,[374265]=109,[319230]=98,[383269]=147,[373923]=102,[276837]=84,[405166]=172,[356367]=101,[377440]=133,[377592]=140,[377590]=139,[377585]=137,[373930]=104,[374504]=112,[281238]=88,[48743]=10,[377580]=136,[207272]=66,[377629]=141,[377376]=132,[377351]=131,[377056]=123,[377226]=130,[389679]=148,[115989]=34,[42650]=1,[316941]=94,[45524]=2,[49020]=12,[317133]=95,[59057]=26,[49028]=13,[46584]=3,[276079]=83,[317198]=96,[389682]=149,[316867]=92,[85948]=30,[374574]=113,[374383]=111,[376905]=120,[49206]=16,[194844]=40,[275699]=81,[316838]=91,[374277]=110,[114556]=33,[377190]=129,[390259]=155,[378848]=146,[377637]=142,[390196]=153,[207104]=58,[48792]=11,[46585]=4,[317610]=97,[63560]=27,[374261]=108,},
	[872] = {[376237]=113,[369990]=35,[370819]=44,[376239]=114,[370821]=45,[376240]=115,[406904]=148,[359793]=7,[370886]=50,[403208]=141,[368847]=24,[375796]=101,[371270]=61,[376179]=108,[370888]=51,[375797]=102,[371016]=56,[409267]=161,[412710]=177,[375544]=85,[369297]=26,[407866]=152,[375801]=103,[412713]=178,[381922]=125,[387341]=134,[407869]=153,[372233]=66,[369939]=34,[376888]=117,[370960]=53,[370897]=52,[370962]=54,[360827]=10,[375554]=86,[375618]=91,[370837]=46,[407876]=154,[370455]=38,[373834]=70,[377086]=120,[370839]=47,[412723]=179,[374346]=74,[371032]=57,[410685]=170,[374348]=75,[385696]=126,[371034]=58,[410687]=171,[375561]=88,[408775]=160,[386272]=127,[359816]=8,[370845]=48,[351338]=1,[365937]=18,[376138]=104,[373270]=69,[375757]=97,[376204]=109,[412733]=180,[371806]=63,[386405]=133,[386342]=130,[377099]=121,[376207]=110,[371426]=62,[377100]=122,[375443]=79,[357211]=4,[404195]=143,[414969]=182,[368432]=22,[376210]=111,[386283]=128,[370537]=39,[370665]=41,[375574]=89,[386348]=131,[408543]=159,[412774]=181,[370783]=43,[411212]=176,[411164]=174,[376150]=105,[410787]=173,[376872]=116,[375577]=90,[367226]=20,[408004]=156,[410352]=168,[410261]=167,[371038]=59,[410253]=165,[409676]=164,[369459]=27,[409329]=163,[363916]=14,[418101]=183,[410260]=166,[408083]=157,[410643]=169,[375517]=81,[403631]=142,[408002]=155,[407814]=151,[369908]=31,[407243]=150,[378196]=123,[406907]=149,[363534]=13,[406732]=147,[406659]=146,[369846]=28,[375520]=82,[405659]=145,[359073]=6,[370867]=49,[387761]=135,[355936]=2,[396286]=140,[368412]=21,[375777]=98,[374968]=76,[395152]=137,[387787]=136,[369913]=33,[372527]=68,[409311]=162,[370062]=36,[358385]=5,[375510]=80,[404977]=144,[381921]=124,[395160]=138,[386336]=129,[370553]=40,[375542]=84,[374251]=73,[366155]=19,[376164]=106,[362874]=12,[373861]=71,[370781]=42,[376930]=118,[369089]=25,[375783]=99,[371257]=60,[376166]=107,[411165]=175,[372469]=67,[408233]=158,[371832]=64,[375721]=92,[357170]=3,[410784]=172,[372048]=65,[375722]=95,[364343]=15,[365933]=17,[365585]=16,[368838]=23,[396187]=139,[360806]=9,[375556]=87,[377082]=119,[360995]=11,[375087]=77,[375406]=78,[375725]=96,[375528]=83,[369909]=32,[374227]=72,[376236]=112,[370452]=37,},
	[790] = {[152261]=29,[404542]=183,[406646]=199,[378405]=84,[383314]=100,[379043]=93,[414170]=212,[384909]=111,[216331]=56,[231663]=62,[31935]=16,[386568]=130,[405757]=190,[393071]=159,[403654]=171,[385422]=119,[387781]=137,[404357]=179,[403975]=177,[384914]=112,[1044]=3,[378285]=83,[414113]=209,[96231]=23,[157047]=31,[85043]=20,[385427]=121,[387786]=138,[20066]=6,[383388]=105,[10326]=5,[200430]=42,[384027]=107,[406468]=197,[53376]=17,[403664]=172,[383328]=101,[209389]=52,[31884]=15,[416799]=223,[230332]=59,[406154]=194,[414378]=217,[393022]=156,[383396]=106,[414443]=218,[20473]=7,[231895]=63,[378425]=85,[393024]=157,[406157]=195,[392961]=155,[53385]=18,[414127]=210,[402971]=163,[416869]=224,[183415]=32,[85222]=21,[385633]=125,[387801]=139,[379021]=91,[388018]=146,[114165]=26,[405461]=188,[385125]=113,[287269]=67,[383342]=102,[402912]=161,[405289]=186,[31821]=9,[231644]=61,[414708]=219,[326732]=72,[406101]=193,[387805]=140,[414273]=216,[379008]=90,[403042]=166,[200025]=41,[414214]=215,[326734]=73,[387998]=144,[392907]=150,[414203]=214,[184662]=36,[204074]=50,[387808]=141,[414195]=213,[414139]=211,[385450]=122,[414073]=208,[414065]=207,[404512]=181,[409441]=206,[385515]=124,[24275]=8,[86659]=22,[406872]=201,[392911]=151,[384815]=108,[200474]=43,[197446]=39,[385349]=115,[408459]=204,[255937]=65,[404834]=184,[407067]=203,[406064]=192,[406835]=200,[204077]=51,[387814]=142,[406545]=198,[378762]=87,[406158]=196,[406940]=202,[387879]=143,[405768]=191,[388007]=145,[375576]=77,[384820]=109,[210294]=53,[377043]=79,[403745]=175,[403026]=165,[414720]=220,[389539]=147,[383228]=99,[183998]=34,[321136]=71,[405278]=185,[404540]=182,[327193]=74,[379022]=92,[404436]=180,[404306]=178,[403826]=176,[53595]=19,[405607]=189,[402964]=162,[392902]=149,[31850]=10,[386738]=133,[403530]=170,[403696]=173,[403495]=168,[305394]=69,[1022]=2,[204018]=45,[403479]=167,[405355]=187,[415364]=221,[403698]=174,[183778]=33,[184575]=35,[393114]=160,[204019]=46,[393030]=158,[392951]=154,[392938]=153,[6940]=4,[403509]=169,[377053]=80,[391142]=148,[382536]=98,[416770]=222,[409439]=205,[392928]=152,[378457]=86,[200652]=44,[417855]=225,[385425]=120,[377128]=81,[384897]=110,[385726]=127,[198034]=40,[148039]=28,[234299]=64,[115750]=27,[385639]=126,[231642]=60,[204054]=49,[385728]=129,[386653]=131,[204023]=47,[223817]=58,[633]=1,[385464]=123,[343527]=75,[376996]=78,[378845]=88,[315924]=70,[196926]=38,[378279]=82,[379391]=96,[387170]=135,[378974]=89,[190784]=37,[156910]=30,[343721]=76,[385129]=114,[385414]=118,[215661]=55,[387174]=136,[383344]=103,[403010]=164,[114158]=25,[380188]=97,[114154]=24,[213644]=54,[280373]=66,},
	[786] = {[381655]=126,[375982]=100,[382039]=152,[17364]=13,[5394]=5,[381785]=138,[210689]=77,[198067]=65,[333974]=92,[60188]=26,[273221]=86,[200076]=71,[428071]=205,[386443]=195,[381726]=133,[77472]=31,[117013]=44,[188443]=51,[382046]=157,[192077]=57,[378094]=106,[305483]=88,[384087]=182,[382685]=167,[108280]=37,[381666]=128,[117014]=46,[210853]=79,[334046]=94,[333919]=91,[114050]=41,[108281]=38,[33757]=15,[198838]=68,[51886]=22,[383010]=173,[382309]=164,[382947]=171,[114051]=42,[187874]=49,[30884]=14,[383012]=175,[197214]=63,[334308]=96,[384352]=185,[201900]=72,[381930]=142,[381867]=141,[114052]=43,[382888]=170,[381932]=143,[383016]=177,[382315]=165,[381678]=129,[383017]=178,[381743]=134,[383019]=179,[60103]=25,[381936]=144,[8042]=6,[390288]=197,[51505]=18,[1064]=3,[378241]=110,[382194]=160,[187880]=50,[384363]=188,[157153]=47,[73685]=29,[157154]=48,[428332]=206,[377933]=101,[334195]=95,[192249]=60,[404522]=203,[192058]=55,[207399]=74,[404015]=201,[393905]=200,[51490]=17,[392352]=199,[196840]=61,[51514]=19,[381819]=140,[382197]=161,[383222]=180,[384450]=193,[378776]=121,[386474]=196,[16191]=11,[260878]=81,[108271]=36,[108287]=40,[382033]=151,[378443]=119,[384411]=190,[384405]=189,[384359]=187,[8512]=8,[384355]=186,[16166]=10,[384149]=184,[384143]=183,[61882]=28,[378271]=117,[383013]=176,[383011]=174,[378255]=111,[383009]=172,[382886]=169,[108285]=39,[198103]=66,[192222]=59,[378077]=103,[378193]=107,[16196]=12,[382201]=162,[390370]=198,[381764]=135,[382191]=159,[207778]=76,[192088]=58,[342240]=97,[382045]=156,[382042]=155,[382040]=153,[382021]=147,[57994]=23,[384444]=191,[191861]=54,[378310]=118,[382086]=158,[319930]=89,[382020]=146,[381946]=145,[265046]=85,[191634]=53,[382215]=163,[192063]=56,[378773]=120,[384447]=192,[197995]=64,[381787]=139,[404019]=202,[262303]=82,[381707]=131,[381689]=130,[77756]=33,[210714]=78,[381708]=132,[382027]=148,[378266]=112,[370]=1,[262624]=83,[383303]=181,[236501]=80,[51485]=16,[58875]=24,[8143]=7,[378268]=113,[381647]=123,[974]=2,[382030]=149,[378779]=122,[207401]=75,[382732]=168,[381776]=136,[378270]=116,[378079]=104,[98008]=35,[382032]=150,[79206]=34,[381650]=125,[280614]=87,[200071]=69,[344357]=98,[378081]=105,[198434]=67,[196884]=62,[385923]=194,[51533]=20,[262647]=84,[200072]=70,[404539]=204,[378075]=102,[382482]=166,[378211]=109,[73920]=30,[334033]=93,[61295]=27,[381782]=137,[204268]=73,[51564]=21,},
	[720] = {[196408]=34,[324536]=67,[387075]=105,[427684]=174,[387522]=137,[265187]=52,[386185]=80,[386759]=94,[386951]=99,[387972]=145,[387016]=103,[270545]=62,[386124]=78,[199471]=38,[17962]=9,[288843]=64,[231793]=46,[196412]=35,[264874]=51,[266086]=53,[267170]=55,[387084]=108,[267171]=56,[405955]=163,[111898]=25,[386194]=81,[205148]=40,[205180]=42,[387279]=125,[386833]=95,[196447]=36,[387153]=113,[63106]=15,[387600]=143,[387602]=144,[386646]=88,[386200]=82,[264119]=49,[387093]=109,[48181]=13,[219272]=45,[386648]=89,[405327]=158,[387541]=139,[387095]=110,[205184]=43,[1122]=3,[389775]=155,[428344]=175,[5484]=4,[5740]=5,[416615]=168,[116858]=26,[196102]=29,[387483]=133,[387165]=116,[196103]=30,[104316]=17,[387549]=140,[387103]=111,[27243]=10,[198590]=37,[264130]=50,[334319]=70,[171975]=28,[386659]=90,[6789]=7,[278350]=63,[387108]=112,[386344]=84,[387173]=118,[387301]=126,[386664]=91,[388832]=148,[387494]=134,[387176]=119,[603]=1,[108558]=22,[386858]=96,[386922]=98,[386986]=101,[108415]=18,[428394]=177,[334585]=71,[428351]=176,[264078]=48,[426115]=173,[422057]=172,[422054]=171,[316099]=65,[417234]=170,[416621]=169,[416341]=167,[416230]=166,[416183]=165,[416154]=164,[405936]=162,[108416]=19,[405670]=160,[386864]=97,[405798]=161,[405330]=159,[399668]=157,[387976]=146,[333889]=69,[389630]=154,[389609]=153,[389590]=152,[389576]=151,[56244]=14,[389359]=149,[32388]=12,[388827]=147,[387250]=120,[387569]=141,[386613]=85,[389992]=156,[17877]=8,[387275]=124,[387526]=138,[268358]=61,[386997]=102,[267211]=57,[386105]=75,[710]=2,[215941]=44,[387506]=135,[387475]=132,[387445]=131,[387509]=136,[387252]=121,[386617]=86,[387399]=130,[386110]=76,[267214]=58,[264000]=47,[80240]=16,[387159]=115,[387065]=104,[384069]=72,[386976]=100,[205145]=39,[267216]=59,[386620]=87,[386174]=79,[6353]=6,[267217]=60,[387322]=127,[196277]=32,[266134]=54,[387578]=142,[387273]=123,[386686]=92,[387384]=128,[328774]=68,[386113]=77,[111400]=23,[152108]=27,[387259]=122,[30283]=11,[196406]=33,[387166]=117,[205179]=41,[196226]=31,[386689]=93,[317138]=66,[389367]=150,[387391]=129,[387156]=114,[385899]=74,[386256]=83,[108503]=21,[385881]=73,[111771]=24,},
}

local treeNodeEntryIDs = {
	[850] = {[112182]=386196,[112198]=107570,[112214]=64382,[112230]=275336,[112246]=382948,[112262]=383468,[112278]=392536,[112294]=388933,[112310]=383154,[112326]=383103,[112119]=385512,[112135]=383287,[112151]=12975,[112167]=871,[114733]=400205,[112199]=382940,[112215]=384110,[112231]=394307,[112247]=376079,[112263]=208154,[112279]=206315,[112295]=315720,[112311]=383703,[112327]=384042,[112136]=772,[112152]=6572,[112168]=281001,[112184]=386164,[112200]=382940,[112216]=392792,[112232]=107574,[112248]=392777,[112264]=184364,[112280]=388004,[112296]=383297,[112312]=390563,[114639]=390713,[112121]=388807,[112137]=262150,[112153]=236279,[112169]=386027,[112185]=262231,[112201]=382956,[112217]=382549,[112233]=384124,[112249]=103827,[112265]=85288,[112281]=1719,[112297]=388049,[112313]=386628,[112122]=12294,[112138]=383219,[112154]=384361,[112170]=202743,[112186]=3411,[112202]=384090,[112218]=382461,[112234]=391270,[112250]=275338,[112266]=383852,[112282]=383922,[112298]=12950,[112314]=227847,[114641]=383155,[112123]=7384,[112139]=262161,[112155]=394311,[112171]=386011,[112187]=386208,[112203]=203201,[112219]=202163,[112235]=382939,[112251]=382954,[112267]=383848,[112283]=389603,[112299]=392931,[112315]=383338,[114642]=400314,[112108]=384124,[112124]=202316,[112140]=383442,[112156]=394062,[112172]=386328,[114738]=190456,[112204]=384277,[112220]=383762,[112236]=390642,[112252]=5246,[112268]=215568,[112284]=396749,[112300]=280721,[114643]=386208,[112109]=394855,[112125]=316405,[112141]=385008,[112157]=384041,[112173]=385952,[114739]=400803,[112205]=6343,[112221]=391572,[112237]=392790,[112253]=23920,[112269]=81099,[112285]=152278,[112301]=316402,[112317]=389306,[114644]=386208,[112110]=392966,[112126]=29725,[112142]=248621,[112158]=202560,[112174]=384067,[114740]=386630,[112206]=382258,[112222]=384969,[112238]=382260,[112254]=346002,[112270]=385703,[112286]=383459,[112302]=393965,[112318]=386634,[112111]=29725,[112127]=383103,[112143]=152278,[112159]=1160,[112175]=202603,[112191]=383115,[112207]=316733,[112223]=384318,[112239]=18499,[112255]=392936,[112271]=383959,[112287]=394329,[112303]=382953,[112319]=383292,[112112]=386164,[112128]=118038,[112144]=167105,[112160]=386034,[112176]=203177,[112192]=384404,[112208]=6544,[112224]=382946,[112240]=382900,[114296]=396719,[112272]=383486,[112288]=391683,[112304]=228920,[112320]=383341,[112113]=394312,[112129]=384361,[112145]=281001,[112161]=386071,[112177]=385843,[112193]=383115,[112209]=382764,[112225]=391271,[112241]=275339,[112257]=390563,[112273]=383885,[112289]=385059,[112305]=384063,[112321]=385704,[112130]=316440,[112146]=383430,[112162]=385840,[112178]=390725,[112194]=384404,[112210]=12323,[112226]=390135,[112242]=46968,[112258]=382953,[112274]=393950,[112290]=383916,[112306]=202095,[112322]=393967,[114769]=401150,[112115]=280001,[112131]=385571,[112147]=845,[112163]=1161,[112179]=382896,[112195]=383115,[112211]=384100,[112227]=390123,[112243]=391997,[112259]=383854,[112275]=335077,[112291]=388903,[112307]=386394,[112323]=275334,[112256]=228920,[114295]=396719,[112116]=386477,[112132]=386357,[112148]=383293,[112164]=384072,[112180]=386285,[112196]=384404,[112212]=382764,[112228]=390140,[112244]=390675,[112260]=280392,[112276]=383877,[112292]=383295,[112183]=202168,[112324]=385888,[114293]=383082,[114294]=390674,[112117]=383317,[112133]=260643,[112149]=190456,[112165]=397103,[112181]=382895,[112197]=390354,[112213]=215571,[112229]=390138,[112245]=382767,[112261]=23881,[112277]=184367,[112293]=383605,[112309]=389308,[112325]=384036,[112188]=97462,[112189]=29838,[112118]=334779,[112134]=184783,[112150]=386030,[112166]=152278,[114770]=107574,},
	[774] = {[100532]=385737,[100548]=294029,[100564]=260248,[100580]=257621,[100596]=193533,[100612]=407412,[100628]=375891,[100644]=199921,[100660]=378442,[100676]=199532,[100692]=393344,[100517]=389019,[100533]=385718,[100549]=378916,[100565]=263186,[100581]=204089,[100597]=321460,[100613]=269737,[100629]=201430,[100645]=378010,[100661]=267116,[100677]=424558,[100518]=388057,[100534]=400456,[100550]=378934,[100566]=259387,[100582]=260240,[100598]=260367,[100614]=273887,[100630]=2643,[100646]=266921,[100662]=378209,[100678]=378244,[100519]=388056,[100535]=384790,[100551]=186270,[100567]=265895,[100583]=378771,[100599]=378905,[100615]=271788,[100631]=378002,[100647]=199483,[100663]=193455,[100679]=257944,[100520]=388045,[100536]=384799,[100552]=212436,[100568]=259495,[100584]=260228,[100600]=321293,[100616]=5116,[100632]=343244,[100648]=34026,[100664]=424557,[100680]=193532,[100521]=388042,[100537]=320976,[100553]=187708,[100569]=260331,[100585]=257044,[100601]=378767,[100617]=19801,[100633]=321468,[100649]=343242,[100665]=378210,[100681]=185789,[100522]=388039,[100538]=53351,[100554]=260285,[100570]=360952,[100586]=378888,[100602]=391559,[100618]=162488,[100634]=109215,[100650]=109248,[100666]=393933,[100682]=359844,[100523]=264735,[100539]=53351,[100555]=321290,[100571]=360966,[100587]=288613,[100603]=407404,[100619]=385539,[100635]=378004,[100651]=213691,[100667]=378743,[100683]=217200,[100524]=231548,[100540]=147362,[100556]=378951,[100572]=264332,[100588]=378769,[100604]=321018,[100620]=236776,[100636]=343247,[100652]=392060,[100668]=378207,[100525]=321530,[100541]=34026,[100557]=203415,[100573]=378940,[100589]=194595,[100605]=406425,[100621]=19577,[100637]=34477,[100653]=378740,[100669]=19574,[100510]=389880,[100526]=120360,[100542]=259489,[100558]=378953,[100574]=378962,[100590]=392060,[100606]=378765,[100622]=260241,[100638]=270581,[100654]=378745,[100670]=115939,[100511]=389865,[100527]=385810,[100543]=187707,[100559]=378955,[100575]=271014,[100591]=321287,[100607]=389449,[100623]=378014,[100639]=378007,[100655]=378750,[100671]=56315,[100512]=389866,[100528]=385739,[100544]=257620,[100560]=378961,[100576]=155228,[100592]=378907,[100608]=386878,[100624]=147362,[100640]=1513,[100656]=199530,[100672]=321014,[115478]=53480,[100513]=389882,[100529]=385695,[100545]=269751,[100561]=378950,[100577]=186387,[100593]=378766,[100609]=260404,[100625]=260309,[100641]=187698,[100657]=131894,[100673]=120679,[100514]=390231,[100530]=268501,[100546]=190925,[100562]=186289,[100578]=19434,[100594]=405804,[100610]=378913,[100626]=212431,[100658]=191384,[100674]=199528,[100515]=386870,[100531]=385709,[100547]=378948,[100563]=378937,[100579]=400472,[100595]=260243,[100611]=378910,[100627]=342049,[100643]=343248,[100659]=378205,[100675]=392053,},
	[852] = {[112517]=121411,[112533]=428377,[112549]=196938,[112565]=385408,[112581]=14062,[112597]=58423,[112613]=426563,[112629]=319949,[112645]=381619,[112661]=381669,[112677]=400783,[112518]=394983,[112534]=381877,[112566]=381894,[117172]=426555,[112598]=319951,[112614]=121471,[112630]=5938,[112646]=378803,[112662]=360194,[112678]=381798,[112519]=381624,[112535]=381988,[112551]=271877,[112567]=394321,[112583]=36554,[112599]=428486,[112615]=108209,[112631]=1776,[112647]=354897,[112663]=381800,[112679]=328085,[112520]=79134,[112552]=386823,[112568]=14161,[112616]=382503,[112632]=79008,[112648]=381623,[112664]=381652,[112505]=381664,[112521]=378436,[112537]=381839,[112553]=381846,[112569]=256188,[112585]=31224,[112601]=382017,[112633]=231719,[112649]=14190,[112665]=381634,[114737]=31230,[112506]=385424,[112522]=319066,[112538]=381989,[117144]=423662,[112586]=257505,[112602]=385722,[112618]=382528,[112634]=193546,[112650]=280716,[112507]=385408,[112523]=394332,[112539]=381990,[117145]=423647,[112571]=279876,[112587]=200758,[112603]=280719,[112619]=382524,[112635]=378427,[112651]=14983,[112667]=381802,[112508]=255989,[112524]=381622,[112540]=395422,[117146]=423683,[117162]=196937,[112588]=382507,[112604]=277925,[112620]=245687,[112636]=231691,[112652]=381542,[112668]=381799,[112509]=381640,[112525]=385616,[112541]=381982,[112557]=381828,[112589]=394309,[112605]=382506,[112621]=382514,[117753]=428387,[112669]=385478,[112510]=381626,[112526]=378996,[112542]=256170,[112558]=235484,[112574]=57934,[112590]=382513,[112606]=384631,[112622]=382515,[112638]=400804,[112654]=378813,[112670]=381673,[112511]=392384,[112527]=381845,[117133]=381631,[112559]=381878,[112575]=378807,[112591]=382508,[112607]=382504,[112623]=393972,[112639]=382245,[112655]=381637,[112528]=381885,[117134]=196861,[112560]=196922,[112576]=108208,[112592]=382512,[112608]=382525,[112624]=196976,[112640]=382238,[112656]=5761,[112672]=200806,[117139]=51667,[112513]=319032,[112529]=256165,[112545]=13750,[117151]=423701,[112577]=185313,[112593]=382015,[112609]=382517,[112625]=382505,[117136]=381627,[112657]=5277,[112673]=381632,[117137]=421975,[117135]=381630,[117132]=152152,[117131]=381801,[117130]=385627,[117129]=423136,[112530]=196937,[112546]=381822,[117152]=381621,[117168]=426594,[112594]=382523,[112610]=394320,[112626]=394023,[112642]=193531,[112555]=382746,[117148]=423703,[117173]=131511,[117143]=379005,[112556]=272026,[117740]=200733,[112572]=2094,[112515]=381629,[112531]=108216,[112547]=344363,[112563]=382742,[117169]=426591,[112595]=382518,[112611]=382511,[112627]=343160,[112643]=193539,[112659]=381797,[117171]=385408,[117150]=381543,[112579]=393970,[112580]=381620,[112562]=383281,[112578]=91023,[117106]=423054,[112532]=61329,[112548]=35551,[112564]=424044,[117170]=277953,[112554]=381985,[112612]=185314,[112628]=193537,[112644]=196924,[112660]=255544,[112676]=2823,[117149]=51690,},
	[854] = {[112947]=389693,[112852]=320770,[112868]=389705,[112884]=207697,[112900]=393827,[117761]=428607,[112932]=391275,[112948]=343311,[112837]=370965,[112853]=198793,[112869]=320341,[112885]=263642,[112901]=389708,[112917]=391409,[112933]=389977,[112949]=203550,[112838]=388111,[112870]=263648,[112886]=389997,[112902]=390808,[112918]=389695,[112934]=390154,[112950]=388108,[112823]=428232,[112839]=395446,[112871]=343014,[112887]=207548,[112903]=390213,[117764]=388106,[112951]=320413,[112824]=258881,[112840]=320635,[112872]=389220,[112888]=227174,[112904]=202137,[117765]=390142,[112936]=389978,[115247]=196555,[112825]=390158,[112841]=320386,[112873]=207739,[112889]=389711,[112905]=391178,[112921]=196718,[112953]=343206,[112826]=391189,[112858]=320418,[112890]=391165,[112906]=321028,[117767]=320412,[112938]=428241,[112954]=388116,[112827]=388107,[112859]=207684,[112875]=428603,[112891]=343207,[112907]=389958,[112923]=213410,[112939]=198013,[112955]=258887,[112828]=388114,[112844]=207347,[112860]=389849,[112876]=389732,[112892]=207387,[112908]=212084,[112924]=320331,[112940]=203555,[112956]=258860,[112845]=393822,[112861]=213010,[112877]=336639,[112893]=268175,[117754]=428492,[112925]=320313,[112941]=258876,[112957]=388118,[112830]=320415,[112846]=389696,[112862]=389694,[112878]=389718,[112894]=247454,[117755]=390163,[112926]=278326,[112942]=347461,[112958]=389687,[112831]=388112,[112847]=204909,[112863]=389824,[112879]=389715,[112895]=209258,[112911]=179057,[112927]=217832,[112943]=206476,[112959]=388113,[112832]=320374,[112848]=183782,[112864]=204021,[112880]=326853,[112896]=389985,[117757]=320421,[112928]=320416,[112944]=389688,[112849]=320361,[112865]=389729,[112881]=389721,[112897]=389976,[117758]=209281,[112899]=218612,[112914]=391397,[115248]=205411,[112898]=207407,[112834]=206416,[112850]=389846,[112866]=389724,[112882]=389720,[117743]=328725,[117759]=232893,[115246]=427776,[117748]=427794,[117768]=429211,[117771]=389799,[117763]=342817,[117762]=390152,[112920]=389781,[112912]=389763,[115244]=393029,[112919]=389783,[115245]=343017,[112851]=320654,[112867]=202138,[112883]=320387,[117744]=427640,[117760]=428557,[117741]=427775,[117742]=258925,},
	[793] = {[112963]=197628,[103289]=2637,[103305]=192081,[103321]=102793,[103098]=383192,[103114]=392325,[103130]=426702,[103162]=106951,[103178]=102543,[103194]=393611,[103210]=203965,[103226]=372945,[109872]=393954,[103274]=377801,[103290]=24858,[103306]=16931,[103322]=102359,[103099]=145108,[103115]=203651,[103131]=392116,[103147]=393771,[103163]=202031,[103179]=391875,[103195]=231064,[103211]=50334,[103227]=371999,[109873]=393940,[103275]=252216,[103291]=33786,[103307]=2908,[103323]=29166,[114771]=155580,[103100]=33763,[103116]=207385,[103132]=392099,[103148]=391785,[103164]=48484,[103180]=61336,[103196]=200854,[103212]=203964,[103228]=372618,[109843]=415169,[103225]=203974,[103276]=102401,[103292]=33873,[103308]=288826,[103324]=124974,[103209]=203962,[103101]=132158,[103117]=383191,[103133]=392301,[103149]=384668,[103165]=384667,[103181]=391045,[103197]=135288,[103213]=238049,[103229]=377811,[109853]=383195,[103278]=197626,[103277]=1822,[103293]=2782,[103309]=319454,[103325]=378986,[109861]=202342,[103102]=382550,[103118]=393371,[103134]=392302,[103150]=390864,[103166]=391174,[103182]=384665,[103198]=370586,[103214]=372567,[103230]=155835,[109860]=274281,[109859]=202770,[112968]=197625,[103294]=18562,[103310]=108238,[103326]=377847,[109858]=79577,[103103]=392288,[103119]=391528,[103135]=392160,[103151]=202028,[103167]=16974,[103183]=390772,[103199]=393414,[103215]=384721,[103231]=203953,[116102]=231050,[112966]=197625,[103279]=194153,[103295]=774,[103311]=378988,[109855]=393960,[109856]=390378,[103104]=102351,[103120]=33891,[103136]=197721,[103152]=391881,[103168]=391872,[103184]=285381,[103200]=391528,[103216]=50334,[109862]=202430,[103201]=102558,[114808]=403521,[103280]=78674,[103296]=301768,[103312]=106898,[103232]=345208,[109845]=202918,[103105]=207383,[103121]=392256,[103137]=392410,[103153]=391347,[103169]=391078,[103185]=383352,[109831]=383197,[109847]=394121,[109863]=400059,[114699]=400278,[103208]=158477,[103281]=392378,[103297]=327993,[103313]=377842,[109842]=279620,[109840]=202345,[103106]=392162,[103122]=391951,[103138]=382559,[103154]=391978,[103170]=274837,[103186]=202021,[103202]=391969,[115458]=406890,[109864]=393991,[103219]=377210,[109848]=394048,[103282]=2782,[103298]=22842,[103314]=385786,[117100]=88747,[103220]=372943,[103107]=197073,[103123]=391888,[103139]=382552,[103155]=421432,[103171]=319439,[103187]=16864,[103203]=370695,[109849]=194223,[109865]=394115,[109834]=393956,[109833]=191034,[103283]=2782,[103299]=22570,[103315]=5211,[117105]=400533,[109832]=394046,[103108]=740,[103124]=392315,[103140]=197061,[103156]=400320,[103172]=391972,[103188]=5217,[103204]=393618,[109850]=393760,[103205]=155578,[103221]=204053,[103173]=390902,[103284]=377796,[103300]=1079,[103316]=99,[112965]=197625,[114823]=405300,[103109]=231032,[103125]=392356,[103141]=102342,[103157]=158476,[114648]=202425,[114299]=400142,[109835]=343647,[109851]=394094,[109867]=78675,[103223]=393427,[103222]=80313,[103285]=417712,[103301]=106832,[103317]=159286,[114298]=400140,[103094]=50464,[103110]=200390,[103126]=392124,[103142]=392220,[103158]=391947,[103174]=391951,[103190]=210706,[109836]=394058,[109852]=394065,[109868]=327541,[112964]=197628,[112967]=197628,[103286]=93402,[103302]=106839,[103318]=131768,[117104]=102693,[114809]=289237,[103111]=145205,[103127]=155675,[103143]=113043,[103159]=231063,[103175]=391888,[103191]=6807,[103207]=200851,[114698]=400222,[103206]=371905,[109844]=205636,[114701]=400254,[103287]=132469,[103303]=108299,[103319]=231040,[114300]=400129,[103096]=328025,[103112]=404521,[103128]=392167,[103144]=417710,[103160]=391709,[103176]=391548,[103192]=328767,[109838]=391528,[103224]=50334,[109870]=394081,[109846]=114107,[103218]=372119,[103288]=197524,[103304]=400158,[103320]=48438,[109857]=393958,[103097]=392221,[103113]=158478,[103129]=274902,[103145]=391700,[103161]=236068,[103177]=391528,[103193]=61336,[109839]=102560,[114700]=204066,[109871]=400636,[109841]=202347,},
	[872] = {[115497]=371016,[115513]=410260,[115545]=376138,[115561]=373861,[115577]=369990,[115593]=376872,[115609]=376166,[115625]=386342,[115641]=371038,[115657]=369939,[115498]=395160,[115530]=407876,[115546]=367226,[115562]=376237,[115578]=386272,[115594]=387341,[115610]=371806,[115626]=386336,[115642]=375797,[115658]=374348,[115690]=386405,[115706]=409676,[115499]=369908,[115515]=410685,[115531]=408775,[115547]=377086,[115563]=376236,[115579]=371034,[115595]=370888,[115611]=375520,[115627]=370452,[115643]=375087,[115659]=375577,[115675]=409311,[115500]=408002,[115532]=408004,[115548]=377082,[115564]=368412,[115580]=375618,[115596]=370665,[115612]=370897,[115628]=370455,[115644]=386405,[115660]=375574,[115501]=412733,[115517]=407243,[115533]=404977,[115549]=370960,[115565]=371270,[115581]=359073,[115597]=375406,[115613]=363916,[115629]=411164,[115645]=371016,[115661]=374227,[115502]=396286,[115518]=403208,[115534]=410352,[115550]=377100,[115566]=372233,[115582]=369908,[115598]=375556,[115614]=358385,[115630]=411165,[115646]=375721,[115662]=375561,[115678]=410687,[115519]=375722,[115551]=359793,[115567]=370537,[115583]=370783,[115599]=370886,[115615]=365585,[115631]=375722,[115647]=357211,[115663]=387761,[115679]=410787,[115504]=408543,[115520]=396187,[115536]=403631,[115552]=370062,[115568]=376240,[115584]=405659,[115600]=376930,[115616]=369909,[115632]=375757,[115648]=376204,[115664]=375517,[115680]=410784,[115505]=406904,[115521]=407814,[115537]=410253,[115553]=369908,[115569]=381922,[115585]=368847,[115601]=360806,[115617]=368432,[115633]=370867,[115649]=376207,[115665]=370553,[115506]=408083,[115522]=404195,[115538]=376179,[115554]=373834,[115570]=376210,[115586]=411212,[115602]=374251,[115618]=375510,[115634]=370839,[115650]=357170,[115666]=374968,[115507]=406659,[115523]=407869,[115539]=371426,[115555]=381921,[115571]=376239,[115587]=386283,[115603]=375554,[115619]=376164,[115635]=370845,[115651]=363534,[115667]=365933,[115683]=375777,[115508]=360827,[115540]=375722,[115556]=375783,[115572]=371832,[115588]=375725,[115604]=375443,[115620]=351338,[115636]=370821,[115652]=366155,[115668]=387787,[115493]=408233,[115509]=409329,[115541]=369297,[115557]=373270,[115573]=359816,[115589]=370837,[115605]=368838,[115621]=369913,[115637]=375796,[115653]=364343,[115669]=369459,[115685]=412774,[115510]=410643,[115526]=406732,[115542]=355936,[115558]=372527,[115574]=371257,[115590]=365937,[115606]=374346,[115622]=370819,[115638]=370781,[115654]=372469,[115670]=375544,[115686]=412713,[115495]=410261,[115511]=406907,[115527]=407866,[115543]=362874,[115559]=378196,[115575]=377099,[115591]=369089,[115607]=372048,[115623]=369846,[115639]=386348,[115655]=360995,[115687]=412723,[115496]=395152,[115512]=375796,[115528]=409267,[115544]=376150,[115560]=385696,[115576]=375528,[115592]=370962,[115608]=375542,[115624]=375801,[115640]=376888,[115656]=371032,[115688]=412710,[115881]=414969,[116103]=418101,},
	[795] = {[103767]=596,[103783]=373427,[103799]=375994,[103831]=110744,[103847]=373456,[103863]=321291,[103688]=238135,[103704]=424509,[103720]=390684,[103736]=392988,[103752]=193155,[103768]=196489,[103784]=391228,[103800]=377065,[103816]=391151,[103832]=373466,[103848]=341167,[103864]=32379,[103705]=372969,[103721]=322115,[103737]=235587,[103753]=200199,[103769]=391208,[103785]=407430,[103801]=288733,[103817]=373273,[103833]=390972,[103849]=32375,[103865]=34433,[103674]=228260,[103690]=373003,[103706]=314867,[103722]=194509,[103738]=414553,[103754]=341997,[114735]=373218,[103786]=407470,[103802]=391095,[103818]=390996,[103834]=108942,[103850]=390622,[103866]=393870,[115883]=415479,[103675]=372760,[103691]=246287,[103723]=81749,[103739]=372307,[103755]=64843,[103771]=196707,[103787]=373310,[103803]=73510,[103819]=391112,[103835]=373446,[103851]=132157,[103867]=528,[115884]=415416,[103676]=391124,[103692]=238063,[103708]=373065,[103724]=198068,[103740]=391387,[103756]=391161,[103772]=196437,[103788]=200174,[103804]=341273,[103820]=108968,[103836]=390670,[103852]=390620,[103868]=73325,[103677]=390767,[103693]=215768,[103709]=373054,[103725]=372972,[103741]=391381,[103757]=391209,[103773]=200209,[103789]=375767,[103805]=375888,[103821]=108945,[103837]=375901,[103853]=121536,[103869]=139,[103678]=205364,[103694]=390786,[103710]=123040,[103726]=390686,[103742]=265202,[103758]=204883,[103774]=47788,[103806]=47585,[103822]=373481,[103838]=373223,[103854]=213634,[103870]=33076,[103695]=390770,[103711]=390832,[103727]=47536,[103743]=200183,[103759]=390881,[103775]=2050,[103823]=109186,[103839]=390668,[103855]=390632,[103871]=372354,[103680]=391109,[103696]=390705,[103712]=373427,[103728]=197419,[103744]=390994,[103760]=321377,[103776]=88625,[103792]=15487,[103808]=335467,[103824]=368275,[103840]=199855,[103856]=64129,[103872]=390667,[103681]=392507,[103697]=373180,[103713]=33206,[103729]=390691,[103745]=391368,[103761]=390947,[103777]=372616,[103793]=64044,[103809]=341491,[103825]=238100,[103841]=15286,[103857]=390615,[103873]=377438,[103682]=406777,[103698]=421557,[103714]=373035,[103746]=391339,[103762]=390954,[103778]=391154,[103794]=263716,[103810]=391284,[103826]=373450,[103842]=280749,[103858]=193063,[108225]=231682,[103683]=373212,[103699]=421558,[103715]=372991,[103731]=390693,[103747]=372370,[103763]=390980,[103779]=193157,[103795]=406788,[103811]=155271,[103827]=120644,[103843]=9484,[103859]=108920,[108226]=373049,[103684]=373202,[103700]=421453,[103732]=47515,[103748]=390967,[103764]=196985,[103780]=391233,[103796]=263165,[103812]=391399,[103828]=122121,[103844]=10060,[103860]=390919,[103701]=373042,[103717]=197045,[103733]=372835,[103749]=391186,[103765]=238136,[103781]=373280,[103797]=373221,[103813]=205385,[103829]=373457,[103845]=196704,[103861]=377422,[115449]=391090,[103686]=407469,[103702]=193134,[103718]=204197,[103734]=390992,[103750]=390977,[103766]=34861,[103782]=377349,[103798]=341240,[103814]=199484,[103830]=120517,[103846]=390676,[103862]=605,[114588]=55676,[115448]=238558,[115671]=409044,[103687]=62618,[116182]=271466,[103703]=390689,[103719]=390685,[103735]=372309,[103751]=64901,[103770]=200128,},
	[781] = {[101472]=396166,[101488]=388193,[101504]=116705,[101520]=389578,[101536]=389577,[101552]=386276,[101377]=388615,[101393]=388701,[101409]=387765,[101425]=388854,[101441]=389942,[101457]=387256,[101473]=123904,[101489]=388849,[101505]=344359,[101521]=394123,[101537]=393400,[101362]=196725,[101378]=115310,[101394]=115151,[101410]=116680,[101426]=116645,[101442]=383785,[101458]=122281,[101474]=152175,[101490]=392985,[101506]=115078,[101522]=122278,[101538]=397251,[101363]=197900,[101379]=197915,[101395]=115869,[101411]=196736,[101427]=280197,[101443]=387230,[101459]=387625,[101475]=323999,[101491]=392983,[101507]=116841,[101523]=388809,[101539]=387184,[101364]=388564,[101380]=422031,[101396]=325197,[101412]=387046,[101428]=152173,[101444]=387638,[101460]=325177,[101476]=195300,[101492]=196740,[101508]=107428,[101524]=388674,[101540]=322740,[101365]=388604,[101381]=388212,[101397]=322118,[101413]=388681,[101429]=137639,[101445]=393516,[101461]=383695,[101477]=388661,[101493]=392958,[101509]=115175,[101525]=389574,[101541]=383707,[101366]=388517,[101382]=388031,[101398]=388551,[101414]=264348,[101430]=391370,[101446]=386937,[101462]=322510,[101478]=392993,[101494]=392970,[101510]=231602,[101526]=392910,[101542]=325153,[102626]=218164,[101367]=393460,[101383]=388779,[101399]=388593,[101415]=392994,[101431]=391330,[101447]=383714,[101463]=322507,[101479]=405044,[101495]=116095,[101511]=388664,[101527]=123986,[101543]=387219,[102627]=388874,[101368]=124081,[101384]=388038,[101400]=274963,[101416]=218164,[101432]=101545,[101448]=325093,[101464]=115181,[101480]=392989,[101496]=115203,[101512]=101643,[101528]=115098,[101544]=132578,[101369]=388548,[101385]=388847,[101401]=197895,[101417]=392979,[101433]=388856,[101449]=280515,[101465]=386965,[115761]=414131,[101497]=388813,[101513]=388812,[101529]=388811,[101545]=383700,[114821]=404408,[101370]=124682,[101386]=337209,[101402]=388682,[101418]=393098,[101434]=220357,[101450]=115399,[101466]=383994,[101482]=394923,[101498]=388814,[101514]=328670,[101530]=392900,[101546]=393357,[114822]=392991,[101371]=388740,[101387]=388511,[101403]=116645,[101419]=388846,[101435]=392982,[101451]=121253,[101467]=383697,[101483]=388848,[101499]=387276,[101515]=122783,[101531]=328669,[114297]=399226,[101356]=274909,[101372]=388491,[101388]=343655,[101404]=400089,[101420]=122470,[101436]=116847,[101452]=124502,[101468]=383698,[117804]=122278,[101500]=389575,[101516]=116844,[101532]=115313,[101548]=196730,[116131]=418359,[101357]=387991,[101373]=388661,[101389]=202424,[101405]=400053,[101421]=391383,[101437]=325201,[101453]=119582,[101469]=196737,[117805]=432181,[101501]=322113,[101517]=157411,[101533]=389579,[101549]=116847,[117802]=122278,[101358]=274586,[101374]=122280,[101390]=116849,[101406]=191837,[101422]=115396,[101438]=397768,[115684]=388218,[101470]=322960,[101486]=394093,[101502]=115008,[101518]=196607,[101534]=394110,[101550]=386941,[101547]=115176,[115760]=414132,[117803]=432181,[101359]=388193,[101375]=388509,[101391]=388020,[101407]=388047,[101423]=113656,[115464]=198898,[101455]=322120,[101471]=343743,[101487]=391412,[101503]=115173,[101519]=388686,[101535]=115315,[101551]=386949,[101439]=387035,[101485]=386276,[101481]=392986,[101484]=386941,[101376]=388477,[101392]=399491,[101408]=388023,[101424]=121817,[101440]=324312,[101456]=388505,[114729]=400629,},
	[658] = {[80307]=5143,[80148]=414660,[80164]=383121,[80180]=235450,[80196]=384858,[80212]=270233,[80228]=12982,[80244]=190447,[80260]=44457,[80276]=383499,[80292]=384060,[80308]=153626,[80149]=382800,[80165]=382292,[80181]=45438,[80197]=384612,[80229]=378919,[80245]=205030,[80261]=383391,[80277]=383810,[80293]=321420,[80309]=383661,[80150]=386539,[80166]=343183,[80182]=382424,[80198]=114923,[80214]=382103,[80230]=205027,[80262]=205023,[80278]=343222,[80294]=236628,[80310]=384651,[80151]=383243,[80167]=391102,[80183]=55342,[80215]=417491,[80231]=379993,[80247]=378198,[80279]=383604,[80295]=383782,[80152]=389713,[80168]=382481,[80184]=382820,[80200]=205028,[80216]=205021,[80248]=153595,[80264]=384033,[80280]=269644,[80296]=236457,[116060]=205020,[80153]=382268,[80169]=382289,[80185]=382569,[80201]=231564,[80217]=155149,[80233]=236662,[80249]=378947,[80265]=155148,[80281]=2948,[80297]=321752,[116061]=384174,[80154]=31589,[80170]=382293,[80186]=157997,[80202]=235711,[80218]=417493,[80234]=381706,[80250]=378448,[80266]=383476,[80282]=108853,[80298]=414379,[116062]=383860,[80139]=205026,[80155]=382490,[80171]=382440,[80187]=382493,[80203]=384861,[80219]=382144,[80235]=12472,[80251]=382110,[80267]=203275,[80283]=11366,[80299]=365350,[116063]=416506,[80140]=30449,[80156]=382826,[80188]=235224,[80204]=321387,[80220]=199786,[80236]=378406,[80252]=416719,[80268]=383659,[80284]=157642,[80300]=321526,[80141]=414659,[80157]=386828,[80173]=383092,[80189]=387807,[80205]=383980,[80221]=381244,[80237]=205024,[80253]=153561,[80269]=383967,[80285]=257541,[80301]=321745,[115762]=281482,[80142]=205036,[80158]=382270,[80174]=342245,[80206]=264354,[80222]=378749,[80254]=416715,[80270]=383489,[80286]=387044,[80302]=321507,[80143]=386763,[80159]=342249,[80175]=475,[80191]=342231,[80207]=205022,[80223]=380154,[80239]=417489,[80255]=383665,[80271]=383634,[80287]=117216,[80303]=384581,[80144]=113724,[80160]=157981,[80176]=11426,[80192]=759,[80208]=205025,[80224]=56377,[80256]=205029,[80272]=383669,[80288]=235870,[80304]=376103,[80145]=389627,[80161]=382297,[115877]=110959,[80193]=384276,[80209]=12051,[80225]=379049,[80241]=30455,[80257]=416094,[80273]=383886,[80289]=342344,[80305]=384683,[116101]=417467,[80162]=108839,[115878]=414664,[80194]=384374,[80210]=384187,[80226]=278309,[80242]=84714,[80258]=343230,[80274]=416714,[80290]=157980,[102429]=385167,[102428]=378756,[80178]=235313,[80147]=31661,[80163]=212653,[80179]=390218,[80195]=384452,[116133]=418733,[80227]=112965,[80243]=44614,[80259]=205037,[80275]=190319,[80291]=383676,[80177]=1463,},
	[750] = {[96165]=253593,[96181]=343294,[96197]=48263,[96213]=48792,[96229]=47568,[96245]=49143,[96261]=377668,[96277]=219786,[96293]=390279,[96309]=195679,[96325]=46584,[96166]=391477,[96182]=206967,[96198]=391546,[96214]=373923,[96230]=377190,[96246]=49020,[96262]=377655,[96278]=194679,[96294]=152280,[96310]=276023,[96326]=85948,[96167]=391386,[96183]=374261,[96199]=48707,[96215]=373930,[96231]=207057,[96247]=51128,[96263]=205723,[96279]=195292,[96295]=115989,[96311]=49206,[96327]=55090,[96168]=391517,[96184]=356367,[96200]=49998,[96216]=207104,[96232]=376251,[96248]=376905,[96264]=114556,[96280]=317610,[96296]=207317,[96312]=377514,[96328]=77575,[96169]=391458,[96185]=273952,[96201]=46585,[96217]=391566,[96233]=316803,[96249]=317198,[96265]=206970,[96281]=390236,[96297]=377585,[96313]=390275,[96329]=316867,[96170]=374715,[96186]=276079,[96202]=316916,[96218]=405166,[96234]=51271,[96250]=377076,[96266]=221536,[96282]=390196,[96298]=207269,[96314]=49530,[96330]=390161,[96171]=273953,[96187]=374265,[96203]=327574,[96235]=207142,[96251]=377073,[96267]=108199,[96283]=390259,[96299]=207264,[96315]=152280,[96331]=316941,[96172]=207167,[96188]=111673,[96204]=374049,[96220]=281238,[96236]=377226,[96252]=207061,[96268]=273946,[96284]=377440,[96300]=374737,[96316]=319230,[96332]=390268,[96173]=378848,[96189]=392566,[96205]=374111,[96221]=194913,[96253]=377098,[96269]=49028,[96285]=207289,[96301]=377629,[96317]=194916,[96333]=42650,[96174]=205727,[96190]=374504,[96206]=48743,[96222]=152279,[96238]=377092,[96254]=377083,[96270]=219809,[96286]=377590,[96302]=206974,[96318]=207272,[96334]=377592,[96175]=373926,[96191]=389679,[96207]=212552,[96223]=377047,[96239]=194912,[96255]=374747,[96271]=206940,[96287]=276837,[96303]=195182,[96319]=390175,[96176]=374383,[96192]=389682,[96208]=374598,[96224]=279302,[96240]=57330,[96256]=391398,[96272]=317133,[96288]=377587,[96304]=206930,[96320]=207311,[96161]=392950,[96177]=383269,[96193]=221562,[96209]=374574,[96225]=207230,[96241]=316838,[96257]=374717,[96273]=194662,[96289]=390283,[96305]=50842,[96321]=377580,[96162]=207126,[96178]=47568,[96194]=51052,[96210]=45524,[96226]=377376,[96242]=377056,[96258]=194844,[96274]=221699,[96290]=390270,[96306]=81136,[96322]=275699,[96163]=281208,[96179]=194878,[96195]=374030,[96211]=47528,[96227]=377351,[96243]=59057,[96259]=377640,[96275]=274156,[96291]=277234,[96307]=391395,[96323]=390166,[96164]=376938,[96180]=391571,[96196]=374277,[96212]=207200,[96228]=305392,[96244]=49184,[96260]=377637,[96276]=206931,[96292]=377537,[96308]=55233,[96324]=63560,},
	[790] = {[102524]=406940,[102540]=183998,[102556]=414720,[102572]=392951,[102588]=385639,[102604]=1022,[102620]=385464,[115673]=409439,[102445]=31850,[115466]=386738,[102477]=393024,[102493]=406064,[115020]=404357,[102541]=414443,[102557]=157047,[102573]=200652,[102589]=234299,[102621]=377053,[115164]=403975,[102430]=204019,[102446]=378762,[102462]=280373,[102478]=377043,[102494]=402912,[102510]=405289,[115021]=405278,[102542]=414378,[102558]=387998,[102574]=200474,[102590]=231663,[102606]=384909,[102622]=385427,[115165]=403509,[102431]=53595,[102447]=389539,[102463]=386568,[102479]=24275,[102511]=406157,[115022]=383342,[102543]=414214,[102559]=414139,[102575]=383388,[102591]=96231,[102623]=10326,[102432]=379022,[102448]=31884,[102464]=406101,[102496]=231895,[102512]=406158,[115023]=403745,[102544]=417855,[102560]=114165,[102576]=387170,[102592]=230332,[102608]=408459,[102624]=376996,[115167]=405757,[102433]=385422,[102449]=378279,[102465]=375576,[102481]=204054,[102497]=31884,[102513]=343721,[115024]=403026,[102545]=85222,[102561]=114158,[102577]=392907,[102593]=31884,[102609]=385425,[102625]=190784,[115168]=379391,[102434]=378457,[102450]=379043,[102466]=387174,[102498]=184575,[102514]=404306,[102530]=404540,[102546]=200430,[102562]=387808,[102578]=53376,[102594]=384914,[115169]=405768,[102435]=378845,[115456]=416869,[102467]=321136,[115488]=382536,[102499]=53385,[102515]=403010,[102547]=392911,[102563]=414170,[102579]=183415,[102595]=384820,[102611]=31884,[102436]=204023,[102452]=393030,[102468]=209389,[115489]=223817,[115483]=406154,[102516]=402971,[102532]=200025,[102548]=31821,[102564]=387781,[102580]=196926,[102596]=183778,[115872]=387801,[115481]=326734,[115482]=385414,[115473]=406646,[115474]=404542,[102437]=378285,[102453]=85043,[102469]=378425,[115490]=223817,[115467]=379391,[115468]=384027,[102533]=156910,[116064]=416770,[102565]=392938,[102581]=414203,[102597]=385515,[115873]=414065,[115457]=406835,[115454]=305394,[115453]=406545,[115443]=403664,[102438]=315924,[102454]=378974,[102470]=385726,[115440]=403495,[115439]=406872,[102518]=403042,[102534]=20473,[102550]=287269,[114828]=404436,[115452]=383396,[102598]=377128,[102614]=385129,[115477]=405607,[115051]=326732,[115043]=255937,[114830]=383344,[102439]=393022,[102455]=152261,[102471]=31935,[114826]=383314,[102503]=403698,[102519]=184662,[102535]=414127,[102551]=210294,[102567]=387879,[102583]=633,[102599]=384897,[115875]=414113,[114824]=403696,[102485]=404834,[102615]=385125,[102526]=403654,[102440]=204074,[102456]=86659,[102472]=378405,[102525]=405355,[102504]=383328,[115438]=387170,[102536]=392961,[102552]=392902,[102568]=216331,[102584]=115750,[102600]=384815,[115876]=414273,[102521]=403826,[115435]=343527,[115874]=416799,[102539]=387814,[114831]=215661,[102457]=386653,[102473]=204077,[115239]=403479,[102505]=405461,[115016]=198034,[102537]=415364,[102553]=387786,[102569]=31884,[102585]=20066,[102601]=385414,[102617]=385728,[115475]=407067,[115882]=148039,[116376]=414073,[115465]=379391,[115447]=406468,[102458]=392928,[102474]=327193,[111886]=204018,[102506]=404512,[102616]=391142,[102538]=388018,[102612]=385450,[102570]=387805,[102586]=385633,[102602]=6940,[102619]=385349,[115479]=305394,[102461]=393071,[114292]=231644,[102475]=383388,[102443]=386738,[102459]=204054,[115480]=385414,[102491]=383228,[115444]=403530,[116183]=388007,[115034]=402964,[102451]=379021,[102571]=231642,[102587]=1044,[102603]=114154,[115879]=414708,[116205]=414195,[102549]=197446,[115672]=409441,[111887]=393114,[102444]=379008,[102460]=380188,[102476]=213644,[116065]=385728,},
	[786] = {[101982]=192063,[101998]=5394,[101807]=333974,[101823]=33757,[101839]=384363,[101855]=61882,[101871]=385923,[101887]=378266,[101919]=200071,[101935]=382021,[101951]=1064,[101967]=378773,[101983]=58875,[101999]=378094,[114813]=383222,[101824]=384352,[101840]=187874,[101856]=378776,[101872]=333919,[101888]=378255,[101904]=77472,[101920]=381946,[101936]=382315,[101952]=198103,[101968]=370,[101984]=381678,[102000]=108281,[101809]=201900,[101825]=319930,[101841]=197214,[101857]=381743,[101873]=262303,[101889]=381726,[101905]=61295,[101937]=197995,[101953]=378077,[101985]=383012,[102001]=381930,[101810]=196884,[101826]=384355,[101842]=207778,[101858]=378241,[101874]=273221,[101890]=382032,[101906]=200076,[101922]=378443,[101938]=378270,[101954]=192088,[101970]=378079,[101986]=383011,[102002]=381867,[101811]=390370,[101827]=210853,[101843]=386443,[101859]=382685,[101875]=381787,[101891]=375982,[101923]=73920,[101939]=382194,[101955]=79206,[101971]=204268,[101987]=108285,[102003]=383010,[114291]=114051,[114817]=404522,[101828]=382042,[101844]=382042,[101860]=191634,[101876]=381785,[101892]=191861,[101924]=236501,[117750]=117014,[101956]=382886,[101972]=51514,[101988]=382033,[102004]=974,[101813]=384444,[101829]=384405,[117910]=381650,[101861]=381936,[101877]=114050,[101893]=381707,[101909]=207401,[101925]=157154,[101941]=382020,[101957]=57994,[101973]=108287,[101989]=383013,[114819]=382947,[101830]=375982,[101846]=77756,[101862]=384087,[101878]=378270,[101894]=77756,[101910]=382039,[101926]=382045,[101942]=114052,[101958]=8143,[101974]=30884,[101990]=382201,[101799]=381666,[101815]=384450,[101831]=428071,[101847]=381764,[101863]=191634,[101879]=16166,[101895]=382030,[101911]=382732,[101927]=382309,[101943]=377933,[101959]=381819,[101975]=51485,[101991]=383019,[101800]=393905,[101816]=378270,[101832]=392352,[101848]=378211,[101864]=210689,[101880]=381782,[101896]=16166,[101912]=108280,[101928]=333919,[101944]=381647,[101960]=265046,[101976]=192077,[101992]=383017,[101801]=384149,[101833]=390288,[101849]=192249,[101865]=378271,[101881]=378310,[101897]=378211,[101913]=98008,[101929]=16191,[101945]=108271,[101961]=192058,[101993]=305483,[101802]=384143,[101818]=334308,[101834]=262647,[101850]=198067,[101866]=117014,[101882]=378268,[101898]=280614,[101914]=382040,[101930]=207399,[101946]=382197,[101962]=378075,[101978]=381650,[101994]=378779,[101803]=8512,[101819]=344357,[101835]=198434,[101851]=381666,[101867]=381708,[101883]=381932,[101899]=51564,[101915]=382046,[101931]=198838,[101947]=382888,[101963]=260878,[101979]=381689,[101995]=51490,[101804]=17364,[101820]=384359,[101836]=262624,[101852]=378193,[101868]=382086,[101884]=192222,[101900]=5394,[101916]=382191,[101932]=200072,[101948]=187880,[101964]=383016,[101980]=381655,[101808]=334195,[114818]=404539,[101812]=334046,[114810]=404019,[101821]=342240,[101837]=384447,[101853]=60188,[101869]=381776,[101885]=117013,[101901]=383009,[101917]=428332,[101933]=157153,[101949]=188443,[101965]=196840,[101981]=382215,[101997]=378081,[101845]=386474,[101805]=60103,[114811]=404015,[101806]=334033,[101822]=383303,[101838]=51533,[101854]=8042,[101870]=210714,[101886]=382027,[101902]=16196,[101918]=73685,[101934]=382482,[101950]=51505,[101966]=51886,[101814]=384411,},
	[720] = {[91527]=416154,[91543]=104316,[91559]=386922,[91575]=196103,[91591]=116858,[91432]=171975,[91448]=386256,[91464]=386613,[91480]=422057,[91496]=205148,[91512]=387600,[91528]=387399,[91544]=426115,[91560]=196226,[91576]=108503,[91592]=5740,[91433]=108415,[91449]=386646,[91465]=389609,[91481]=387173,[91497]=387522,[91513]=387578,[91529]=405798,[91545]=416230,[91561]=405330,[91577]=386976,[91434]=386689,[91450]=384069,[91466]=111771,[91482]=387252,[91498]=387509,[91514]=387526,[91530]=416341,[91546]=386185,[91562]=199471,[91578]=386997,[91419]=389992,[91435]=386620,[91451]=264874,[91467]=386659,[91483]=387156,[91499]=196408,[91515]=267217,[91531]=111898,[91547]=267216,[91563]=399668,[91579]=387016,[91420]=389775,[91436]=386858,[91452]=30283,[91468]=317138,[91484]=108503,[91500]=387384,[91516]=387445,[91532]=416183,[91548]=603,[91564]=405327,[91580]=264000,[91421]=389359,[91469]=385899,[91485]=387259,[91501]=196412,[91517]=387391,[91533]=267170,[91549]=387483,[91565]=32388,[91581]=388827,[91422]=386648,[91438]=288843,[91454]=710,[91470]=387475,[91486]=388832,[91502]=1122,[91518]=387322,[91534]=386200,[91550]=265187,[91566]=198590,[91582]=17877,[91423]=387976,[91439]=333889,[91471]=387084,[91487]=152108,[91503]=416621,[91519]=428394,[91535]=205145,[91551]=387075,[91567]=334319,[91583]=387108,[91424]=389590,[91440]=386113,[91456]=386864,[91472]=266086,[91488]=270545,[91504]=387273,[91520]=196277,[91536]=386194,[91552]=48181,[91568]=108558,[91584]=387103,[91425]=389367,[91441]=268358,[91457]=6789,[91473]=417234,[115459]=56244,[91505]=387084,[91521]=264130,[91537]=264119,[91553]=387065,[91569]=316099,[91585]=387166,[91426]=389576,[91442]=328774,[91458]=5484,[91474]=387569,[115460]=386833,[91506]=387301,[91522]=387541,[91538]=428344,[91554]=205180,[91570]=324536,[91586]=196447,[91427]=386344,[91443]=387972,[91459]=386110,[91475]=387165,[115461]=405936,[91507]=387250,[91523]=405670,[91539]=264078,[91555]=386986,[91571]=27243,[91587]=231793,[91428]=389630,[91444]=108416,[91460]=111400,[91476]=387159,[115462]=405955,[91508]=387084,[91524]=387549,[91540]=267171,[91556]=278350,[91572]=196102,[91588]=205184,[91429]=416615,[91445]=386686,[91461]=386124,[91477]=387153,[91493]=80240,[91509]=427684,[91525]=387602,[91541]=267211,[91557]=205179,[91573]=386759,[91589]=196406,[91430]=386617,[91446]=386664,[91462]=386105,[91478]=387279,[91494]=387506,[91510]=334585,[91526]=387494,[91542]=386174,[91558]=386951,[91574]=63106,[91590]=17962,[91492]=6353,[117751]=422054,[91489]=387095,[91490]=387093,[91491]=387176,[91431]=215941,[91447]=385881,[91463]=219272,[91479]=387275,[91495]=266134,[91511]=267214,[117752]=428351,},
}

local talentIDFix = { [103211]=377779,[103216]=343240,[103224]=377623, }

local talentChargeFix = { [36554]={[259]=1,[261]=1},[191634]=true,[47568]=true,[5394]=true }

local MAX_NUM_TALENTS = MAX_NUM_TALENTS or (E.isWOTLKC and 31 or 25)

local GetSelectedTalentData = (E.isDF and function(info, inspectUnit, isInspect)
	local list, c
	if not isInspect then
		list, c = { CM.SERIALIZATION_VERSION, true, "^T" }, 4
	end

	for i = 1, 3 do
		local talentID
		if isInspect then
			talentID = C_SpecializationInfo_GetInspectSelectedPvpTalent(inspectUnit, i)
		else
			local slotInfo = C_SpecializationInfo_GetPvpTalentSlotInfo(i)
			talentID = slotInfo and slotInfo.selectedTalentID
		end
		if talentID then
			local _,_,_,_,_, spellID = GetPvpTalentInfoByID(talentID)
			info.talentData[spellID] = "PVP"
			if list then
				list[c] = -spellID
				c = c + 1
			end
		end
	end

	local configID = isInspect and Constants.TraitConsts.INSPECT_TRAIT_CONFIG_ID or C_ClassTalents.GetActiveConfigID()
	local configInfo = configID and C_Traits.GetConfigInfo(configID)
	if not configInfo then
		return list
	end
	local treeID = configInfo.treeIDs[1]
	local nodeEntryIDs = treeNodeEntryIDs[treeID]
	local nodeSpellIDs = treeNodeSpellIDs[treeID]
	local treeNodes = C_Traits.GetTreeNodes(treeID)
	for _, treeNodeID in ipairs(treeNodes) do
		local treeNode = C_Traits_GetNodeInfo(configID, treeNodeID)
		local activeEntry = treeNode.activeEntry
		if activeEntry then
			local activeRank = treeNode.activeRank
			if activeRank > 0 then
				local activeEntryID = activeEntry.entryID
				local spellID = talentIDFix[activeEntryID] or (nodeEntryIDs and nodeEntryIDs[activeEntryID])
				if spellID then
					if talentChargeFix[spellID] then
						if talentChargeFix[spellID] == true then
							if info.talentData[spellID] then
								activeRank = 2
							end
						elseif talentChargeFix[spellID][info.spec] then
							activeRank = 2
						end
					end
					info.talentData[spellID] = activeRank
					if list then
						spellID = nodeSpellIDs[spellID] or spellID
						list[c] = activeRank > 1 and format("%s:%s", spellID, activeRank) or spellID
						c = c + 1
					end
				end
			end
		end
	end

	return list
end) or (E.isSL and function(info, inspectUnit, isInspect)
	local list
	if not isInspect then
		list = { CM.SERIALIZATION_VERSION, true, "^T" }
	end

	for i = 1, 3 do
		local talentID
		if isInspect then
			talentID = C_SpecializationInfo_GetInspectSelectedPvpTalent(inspectUnit, i)
		else
			local slotInfo = C_SpecializationInfo_GetPvpTalentSlotInfo(i)
			talentID = slotInfo and slotInfo.selectedTalentID
		end
		if talentID then
			local _,_,_,_,_, spellID = GetPvpTalentInfoByID(talentID)
			info.talentData[spellID] = "PVP"
			if list then list[#list + 1] = -spellID end
		end
	end

	local specGroupIndex = 1
	for tier = 1, MAX_TALENT_TIERS do
		for column = 1, NUM_TALENT_COLUMNS do
			local _,_,_, selected, _, spellID = GetTalentInfo(tier, column, specGroupIndex , isInspect, inspectUnit)
			if selected then
				info.talentData[spellID] = true
				if list then list[#list + 1] = spellID end
				break
			end
		end
	end

	return list
end) or (E.isWOTLKC and function(info, inspectUnit, isInspect)
	local list
	if not isInspect then
		list = { CM.SERIALIZATION_VERSION, true, "^T" }
	end

	if list then
		for i = 1, 6 do
			local _,_, glyphSpellID = GetGlyphSocketInfo(i)
			if glyphSpellID then
				info.talentData[glyphSpellID] = true
				list[#list + 1] = glyphSpellID
			end
		end
	end

	local talentGroup = GetActiveTalentGroup and GetActiveTalentGroup(isInspect, nil)
	for tabIndex = 1, 3 do
		for talentIndex = 1, MAX_NUM_TALENTS do
			local name, _,_,_, currentRank = GetTalentInfo(tabIndex, talentIndex, isInspect, inspectUnit, talentGroup)
			if not name then
				break
			end
			if currentRank > 0 then
				local talentRankIDs = E.talentNameToRankIDs[name]
				if talentRankIDs then
					if type(talentRankIDs[1]) == "table" then
						for _, t in pairs(talentRankIDs) do
							local talentID = t[currentRank]
							if talentID then
								info.talentData[talentID] = true
								if list then list[#list + 1] = talentID end
							end
						end
					else
						local talentID = talentRankIDs[currentRank]
						if talentID then
							info.talentData[talentID] = true
							if list then list[#list + 1] = talentID end
						end
					end
				end
			end
		end
	end

	return list
end) or function(info, inspectUnit, isInspect)
	local list
	if not isInspect then
		list = { CM.SERIALIZATION_VERSION, true, "^T" }
	end

	for tabIndex = 1, 3 do
		for talentIndex = 1, MAX_NUM_TALENTS do
			local name, _,_,_, currentRank = GetTalentInfo(tabIndex, talentIndex, isInspect, inspectUnit)
			if not name then
				break
			end
			if currentRank > 0 then
				local talentRankIDs = E.talentNameToRankIDs[name]
				if talentRankIDs then
					if type(talentRankIDs[1]) == "table" then
						for _, t in pairs(talentRankIDs) do
							local talentID = t[currentRank]
							if talentID then
								info.talentData[talentID] = true
								if list then list[#list + 1] = talentID end
							end
						end
					else
						local talentID = talentRankIDs[currentRank]
						if talentID then
							info.talentData[talentID] = true
							if list then list[#list + 1] = talentID end
						end
					end
				end
			end
		end
	end

	return list
end

function CM:InspectUnit(guid)
	local info = P.groupInfo[guid]
	if not info or self.syncedGroupMembers[guid] then
		ClearInspectPlayer()
		return
	end

	local inspectUnit = info.unit
	local specID = E.preCata and info.raceID or GetInspectSpecialization(inspectUnit)
	if not specID or specID == 0 then
		return
	end
	info.spec = specID
	if info.name == "" or info.name == "Unknown" then
		info.name = GetUnitName(inspectUnit, true)
	end
	if info.level == 200 then
		local lvl = UnitLevel(inspectUnit)
		if lvl > 0 then
			info.level = lvl
		end
	end
	if not E.preCata then
		info.spellHasteMult = 1/(1 + UnitSpellHaste(info.unit)/100)
	end

	wipe(info.talentData)
	wipe(info.itemData)

	GetSelectedTalentData(info, inspectUnit, true)
	local failed = GetEquippedItemData(info, inspectUnit, specID)

	ClearInspectPlayer()
	self:DequeueInspect(guid, failed)
	P:UpdateUnitBar(guid)
end

local enhancedSoulbindRowRenownLevel = {
	[1]  = { [1] = 63, [3] = 66, [5] = 68, [6] = 72, [8] = 73, [10] = 78 },
	[2]  = { [1] = 61, [3] = 64, [5] = 67, [6] = 70, [8] = 75, [10] = 79 },
	[3]  = { [1] = 62, [3] = 65, [5] = 69, [6] = 71, [8] = 74, [10] = 77 },
	[4]  = { [1] = 63, [3] = 66, [5] = 68, [6] = 72, [8] = 73, [10] = 78 },
	[5]  = { [1] = 61, [3] = 64, [5] = 67, [6] = 70, [8] = 75, [10] = 79 },
	[6]  = { [1] = 62, [3] = 65, [5] = 69, [6] = 71, [8] = 74, [10] = 77 },
	[7]  = { [1] = 63, [3] = 66, [5] = 68, [6] = 72, [8] = 73, [10] = 78 },
	[8]  = { [1] = 63, [3] = 66, [5] = 68, [6] = 72, [8] = 73, [10] = 78 },
	[9]  = { [1] = 61, [3] = 64, [5] = 67, [6] = 70, [8] = 75, [10] = 79 },
	[10] = { [1] = 62, [3] = 65, [5] = 69, [6] = 71, [8] = 74, [10] = 77 },
	[13] = { [1] = 61, [3] = 64, [5] = 67, [6] = 70, [8] = 75, [10] = 79 },
	[18] = { [1] = 62, [3] = 65, [5] = 69, [6] = 71, [8] = 74, [10] = 77 },
}

local function IsSoulbindRowEnhanced(soulbindID, row, renownLevel)
	local minLevel = enhancedSoulbindRowRenownLevel[soulbindID] and enhancedSoulbindRowRenownLevel[soulbindID][row]
	if minLevel then
		return renownLevel >= minLevel
	end
end

local function GetCovenantSoulbindData(info, list)
	wipe(info.shadowlandsData)

	local covenantID = C_Covenants.GetActiveCovenantID()
	if covenantID == 0 then
		return
	end

	local covenantSpellID = E.covenant_to_spellid[covenantID]
	info.shadowlandsData.covenantID = covenantID
	info.talentData[covenantSpellID] = "C"
	list[#list + 1] = "^C," .. covenantID

	local soulbindID = C_Soulbinds.GetActiveSoulbindID()
	if soulbindID == 0 then
		return
	end
	info.shadowlandsData.soulbindID = soulbindID
	list[#list + 1] = soulbindID

	local soulbindData = C_Soulbinds.GetSoulbindData(soulbindID)
	local nodes = soulbindData.tree and soulbindData.tree.nodes
	if not nodes then
		return
	end

	local renownLevel = C_CovenantSanctumUI.GetRenownLevel()
	for i = 1, #nodes do
		local node = nodes[i]
		if node.state == Enum.SoulbindNodeState.Selected then
			local conduitID, conduitRank, row, spellID = node.conduitID, node.conduitRank, node.row, node.spellID
			if conduitID ~= 0 then
				spellID = C_Soulbinds_GetConduitSpellID(conduitID, conduitRank)
				if IsSoulbindRowEnhanced(soulbindID, row, renownLevel) then
					conduitRank = conduitRank + 2
				end
				local rankValue = E.soulbind_conduits_rank[spellID] and (E.soulbind_conduits_rank[spellID][conduitRank] or E.soulbind_conduits_rank[spellID][1])
				if rankValue then
					info.talentData[spellID] = rankValue
					list[#list + 1] = format("%s:%s", spellID, rankValue)
				end
			elseif E.soulbind_abilities[spellID] then
				info.talentData[spellID] = 0
				list[#list + 1] = spellID
			end
		end
	end
end

local function FindValidSpellID(info, v)
	if type(v) ~= "table" then
		return info.spec == v or (info.talentData[v] and true)
	end
	if v[1] > 0 then
		for id in pairs(v) do
			if info.talentData[id] then
				return true
			end
		end
	else
		local spellID
		for i = 1, #v, 2 do
			local tid, sid = v[i], v[i + 1]
			tid = i == 1 and -tid or tid
			spellID = info.talentData[tid] and sid
		end
		return spellID or true
	end
end

function CM:UpdateCooldownSyncIDs(info)
	if E.preCata or info.isAdminObsForMDI then return end
	wipe(self.cooldownSyncIDs)
	for id, t in E.pairs(E.sync_cooldowns.ALL, E.sync_cooldowns[E.userClass]) do
		local spellID
		for i = 1, #t do
			local v = t[i]
			spellID = not v or FindValidSpellID(info, v)
			if not spellID then break end
		end
		if spellID then
			self.cooldownSyncIDs[spellID == true and id or spellID] = { 0, -1 }
		end
	end
	self:ToggleCooldownSync()
end

function CM:InspectUser()
	local info = P.userInfo
	local specID
	if E.preCata then
		specID = info.raceID
	else
		local specIndex = GetSpecialization()
		specID = GetSpecializationInfo(specIndex)
	end
	if not specID or specID == 0 then
		return false
	end
	info.spec = specID

	wipe(info.talentData)
	wipe(info.itemData)

	local dataList = GetSelectedTalentData(info, "player")
	GetEquippedItemData(info, "player", specID, dataList)
	if E.postBFA then
		GetCovenantSoulbindData(info, dataList)
		info.spellHasteMult = 1/(1 + UnitSpellHaste("player")/100)

	elseif E.isClassic or E.isBCC then
		local speed = UnitRangedDamage("player")
		if speed and speed > 0 then
			info.rangedWeaponSpeed = speed
			dataList[#dataList + 1] = -speed
		end
	end

	dataList[2] = specID
	local serializedData = concat(dataList, ","):gsub(",%^", "^")
	local compressedData = LibDeflate:CompressDeflate(serializedData)
	local encodedData = LibDeflate:EncodeForWoWAddonChannel(compressedData)
	self.serializedSyncData = encodedData

	self:UpdateCooldownSyncIDs(info)

	if P.groupInfo[E.userGUID] then
		P:UpdateUnitBar(E.userGUID)
	end

	return true
end
